@@ -34,6 +34,7 @@
 #include <string.h>
 #include <assert.h>
 #include "omc_config.h"
+#include "OpenModelicaBootstrappingHeader.h"
 
 #if defined(_MSC_VER) || defined(__MINGW32__)
 #else
@@ -59,11 +60,18 @@ extern char* _replace(char* source_str,char* search_str,char* replace_str); //De
 
 static char* winPath = NULL;
 
-#if defined(linux) || defined(__APPLE_CC__)
+#if !defined(OPENMODELICA_BOOTSTRAPPING_STAGE_1) && (defined(__linux__) || defined(__APPLE_CC__))
 /* Helper function to strip /bin/... or /lib/... from the executable path of omc */
 static void stripbinpath(char *omhome)
 {
-  char *tmp;
+  char *tmp = NULL;
+  /* adrpo: if the path does not contain "bin" or "lib" exit gracefully as otherwise the assertion will trigger */
+  if (strstr(omhome, "bin") == NULL && strstr(omhome, "lib") == NULL)
+  {
+    fprintf(stderr, "could not deduce the OpenModelica installation directory from executable path: [%s], please set OPENMODELICAHOME", omhome);
+    exit(EXIT_FAILURE);
+  }
+
   do {
     assert(tmp = strrchr(omhome,'/'));
     *tmp = '\0';
@@ -73,87 +81,35 @@ static void stripbinpath(char *omhome)
 #endif
 
 /* Do not free or modify the returned variable of getInstallationDirectoryPath. It's part of the environment! */
-#if defined(linux)
-#include <sys/stat.h>
-#include <linux/limits.h>
-#include <unistd.h>
+#if defined(OPENMODELICA_BOOTSTRAPPING_STAGE_1) && !(defined(_MSC_VER) || defined(__MINGW32__))
 const char* SettingsImpl__getInstallationDirectoryPath(void) {
-  struct stat sb;
-  static char omhome[PATH_MAX];
-  static int init = 0;
-  ssize_t r;
-  /* This is bad code using hard-coded limits; but we cannot query the size of symlinks on /proc
-   * because that FS is not POSIX-compliant.
-   */
-  if (init) {
-    return omhome;
-  }
-  r = readlink("/proc/self/exe", omhome, sizeof(omhome)-1);
-  if (r < 0) {
-    perror("readlink");
-    exit(EXIT_FAILURE);
-  }
-  assert(r < sizeof(omhome)-1);
-  omhome[r] = '\0';
-  stripbinpath(omhome);
-  init = 1;
-  return omhome;
+  const char *path = getenv("OPENMODELICAHOME");
+  return path ? path : "OPENMODELICA_BOOTSTRAPPING_STAGE_1_NO_OPENMODELICAHOME";
 }
-#elif defined(__APPLE_CC__)
-
-#if 1
+#elif defined(__linux__) || defined(__APPLE_CC__)
 #include <dlfcn.h>
 
 const char* SettingsImpl__getInstallationDirectoryPath(void) {
   int ret;
-  pid_t pid;
-  static char *omhome;
-  static int init = 0;
-  if (init) {
+  static char *omhome = NULL;
+  if (omhome) {
     return omhome;
   }
 
   Dl_info info;
   if (!dladdr((void*) SettingsImpl__getInstallationDirectoryPath, &info)) {
-    fprintf(stderr, "proc_pidpath() failed: %s\n", strerror(errno));
+    fprintf(stderr, "dladdr() failed: %s\n", strerror(errno));
     exit(EXIT_FAILURE);
   } else {
-    omhome = GC_strdup(info.dli_fname);
+    omhome = omc_alloc_interface.malloc_strdup(info.dli_fname);
     stripbinpath(omhome);
   }
-  init = 1;
-  return omhome;
-}
-
-#else
-/* If we do not use dylib in the future */
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <errno.h>
-#include <libproc.h>
-
-const char* SettingsImpl__getInstallationDirectoryPath(void) {
-  int ret;
-  pid_t pid;
-  static char omhome[PROC_PIDPATHINFO_MAXSIZE];
-  static int init = 0;
-  if (init) {
-    return omhome;
-  }
-
-  pid = getpid();
-  ret = proc_pidpath(pid, omhome, sizeof(omhome));
-  if (ret <= 0) {
-    fprintf(stderr, "proc_pidpath() failed: %s\n", strerror(errno));
+  if (!(omhome && omhome[0])) {
+    fprintf(stderr, "Failed to get binary path from dladdr path: %s\n", info.dli_fname);
     exit(EXIT_FAILURE);
-  } else {
-    stripbinpath(omhome);
   }
-  init = 1;
   return omhome;
 }
-#endif /* dylib */
 
 #else /* Not linux or Apple */
 const char* SettingsImpl__getInstallationDirectoryPath(void) {
@@ -212,9 +168,9 @@ char* Settings_getHomeDir(int runningTestsuite)
   return "%APPDATA%";
 #endif
   if (homePath == NULL || runningTestsuite) {
-    return "";
+    return omc_alloc_interface.malloc_strdup("");
   }
-  return GC_strdup(homePath);
+  return omc_alloc_interface.malloc_strdup(homePath);
 }
 
 // Do not free the returned variable. It's malloc'ed
@@ -237,7 +193,7 @@ char* SettingsImpl__getModelicaPath(int runningTestsuite) {
 #if !(defined(_MSC_VER) || defined(__MINGW32__))
     } else {
       int lenHome = strlen(homePath);
-      buffer = (char*) GC_malloc_atomic(lenOmhome+lenHome+41);
+      buffer = (char*) omc_alloc_interface.malloc_atomic(lenOmhome+lenHome+41);
       snprintf(buffer,lenOmhome+lenHome+41,"%s/lib/omlibrary:%s/.openmodelica/libraries/",omhome,homePath);
     }
 #endif
@@ -247,11 +203,11 @@ char* SettingsImpl__getModelicaPath(int runningTestsuite) {
 #if defined(__MINGW32__) || defined(_MSC_VER)
   /* adrpo: translate this to forward slashes! */
   /* duplicate the path */
-  winLibPath = GC_strdup(path);
+  winLibPath = omc_alloc_interface.malloc_strdup(path);
 
   /* ?? not enough memory for duplication */
   if (!winLibPath)
-    return GC_strdup(path);
+    return omc_alloc_interface.malloc_strdup(path);
 
   /* convert \\ to / */
   while(winLibPath[i] != '\0')
@@ -262,7 +218,7 @@ char* SettingsImpl__getModelicaPath(int runningTestsuite) {
   return winLibPath;
 #endif
 
-  return GC_strdup(path);
+  return omc_alloc_interface.malloc_strdup(path);
 }
 
 static const char* SettingsImpl__getCompileCommand(void)
