@@ -33,41 +33,41 @@
 
 #include "initialization.h"
 
-#include "simulation_data.h"
-#include "util/omc_error.h"
-#include "openmodelica.h"
-#include "openmodelica_func.h"
-#include "simulation/options.h"
-#include "simulation/solver/model_help.h"
+#include "../../../util/omc_error.h"
+#include "../../../util/omc_file.h"
+#include "../../../openmodelica.h"
+#include "../../../openmodelica_func.h"
+#include "../../../simulation/options.h"
+#include "../model_help.h"
 #if !defined(OMC_MINIMAL_RUNTIME)
-#include "util/read_matlab4.h"
+#include "../../../util/read_matlab4.h"
 #endif
-#include "simulation/solver/events.h"
-#include "simulation/solver/stateset.h"
-#include "meta/meta_modelica.h"
+#include "../events.h"
+#include "../stateset.h"
+#include "../../../meta/meta_modelica.h"
 
 #if defined(OMC_NUM_MIXED_SYSTEMS) && OMC_NUM_MIXED_SYSTEMS==0
 #define check_mixed_solutions(X,Y) 0
 #else
-#include "simulation/solver/mixedSystem.h"
+#include "../mixedSystem.h"
 #endif
 
 #if defined(OMC_NUM_LINEAR_SYSTEMS) && OMC_NUM_LINEAR_SYSTEMS==0
 #define check_linear_solutions(X,Y) 0
 #define updateStaticDataOfLinearSystems(X,Y)
 #else
-#include "simulation/solver/linearSystem.h"
+#include "../linearSystem.h"
 #endif
 
 #if defined(OMC_NUM_NONLINEAR_SYSTEMS) && OMC_NUM_NONLINEAR_SYSTEMS==0
 #define check_nonlinear_solutions(X,Y) 0
 #define updateStaticDataOfNonlinearSystems(X,Y)
 #else
-#include "simulation/solver/nonlinearSystem.h"
+#include "../nonlinearSystem.h"
 #endif
 
-#include "simulation/solver/delay.h"
-#include "simulation/solver/synchronous.h"
+#include "../delay.h"
+#include "../synchronous.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -267,6 +267,7 @@ static int symbolic_initialization(DATA *data, threadData_t *threadData)
   {
     long step;
     char buffer[4096];
+    double lambda;
 
     infoStreamPrint(LOG_INIT, 0, "Global homotopy with equidistant step size started.");
 
@@ -276,10 +277,11 @@ static int symbolic_initialization(DATA *data, threadData_t *threadData)
     {
       sprintf(buffer, "%s_equidistant_global_homotopy.csv", mData->modelFilePrefix);
       infoStreamPrint(LOG_INIT, 0, "The homotopy path will be exported to %s.", buffer);
-      pFile = fopen(buffer, "wt");
-      fprintf(pFile, "\"sep=%s\"\n%s", sep, "\"lambda\"");
-      for(i=0; i<mData->nVariablesReal; ++i)
+      pFile = omc_fopen(buffer, "wt");
+      fprintf(pFile, "\"lambda\"");
+      for(i=0; i<mData->nVariablesReal; ++i) {
         fprintf(pFile, "%s\"%s\"", sep, mData->realVarsData[i].info.name);
+      }
       fprintf(pFile, "\n");
     }
 #endif
@@ -288,10 +290,13 @@ static int symbolic_initialization(DATA *data, threadData_t *threadData)
     for(step=0; step<init_lambda_steps; ++step)
     {
       data->simulationInfo->lambda = ((double)step)/(init_lambda_steps-1);
-      infoStreamPrint(LOG_INIT, 0, "homotopy parameter lambda = %g", data->simulationInfo->lambda);
+      lambda = data->simulationInfo->lambda;
+      infoStreamPrint(LOG_INIT, 0, "homotopy parameter lambda = %g", lambda);
 
-      if(data->simulationInfo->lambda > 1.0) {
+      if(data->simulationInfo->lambda > 1.0)
+      {
         data->simulationInfo->lambda = 1.0;
+        lambda = 1.0;
       }
 
       if(0 == step)
@@ -299,12 +304,12 @@ static int symbolic_initialization(DATA *data, threadData_t *threadData)
       else
         data->callback->functionInitialEquations(data, threadData);
 
-      infoStreamPrint(LOG_INIT, 0, "homotopy parameter lambda = %g done\n---------------------------", data->simulationInfo->lambda);
+      infoStreamPrint(LOG_INIT, 0, "homotopy parameter lambda = %g done\n---------------------------", lambda);
 
 #if !defined(OMC_NO_FILESYSTEM)
       if(ACTIVE_STREAM(LOG_INIT))
       {
-        fprintf(pFile, "%.16g", data->simulationInfo->lambda);
+        fprintf(pFile, "%.16g", lambda);
         for(i=0; i<mData->nVariablesReal; ++i)
           fprintf(pFile, "%s%.16g", sep, data->localData[0]->realVars[i]);
         fprintf(pFile, "\n");
@@ -605,7 +610,8 @@ int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod
 
   infoStreamPrint(LOG_INIT, 0, "### START INITIALIZATION ###");
 
-  setAllParamsToStart(data);
+  if (strcmp(pInitMethod, "fmi"))
+    setAllParamsToStart(data);
 
 #if !defined(OMC_MINIMAL_RUNTIME)
   /* import start values from extern mat-file */
@@ -621,7 +627,8 @@ int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod
   }
 #endif
   /* set up all variables with their start-values */
-  setAllVarsToStart(data);
+  if (strcmp(pInitMethod, "fmi"))
+    setAllVarsToStart(data);
 
   if(!(pInitFile && strcmp(pInitFile, ""))) {
     data->callback->updateBoundParameters(data, threadData);
@@ -633,7 +640,7 @@ int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod
   updateStaticDataOfNonlinearSystems(data, threadData);
 
   /* if there are user-specified options, use them! */
-  if (pInitMethod && strcmp(pInitMethod, "")) {
+  if (pInitMethod && (strcmp(pInitMethod, "") && strcmp(pInitMethod, "fmi"))) {
     initMethod = IIM_UNKNOWN;
 
     for (i=1; i<IIM_MAX; ++i) {
