@@ -34,31 +34,55 @@
 #include <float.h>
 #include <math.h>
 
-#include "simulation_data.h"
-#include "openmodelica_func.h"
-#include "util/omc_error.h"
-#include "util/varinfo.h"
+#include "../../openmodelica.h"
+#include "../../simulation_data.h"
+#include "../../openmodelica_func.h"
+#include "../../util/omc_error.h"
+#include "../../util/varinfo.h"
 #include "model_help.h"
-#include "simulation/options.h"
-#include "simulation/simulation_info_json.h"
-#include "util/omc_msvc.h" /* for freaking round! */
+#include "../options.h"
+#include "../simulation_info_json.h"
+#include "../../util/omc_msvc.h" /* for freaking round! */
 #include "nonlinearSystem.h"
 #include "linearSystem.h"
 #include "mixedSystem.h"
 #include "delay.h"
 #include "epsilon.h"
-#include "simulation/solver/stateset.h"
-#include "meta/meta_modelica.h"
+#include "fmi_events.h"
+#include "stateset.h"
+#include "../../meta/meta_modelica.h"
+
+#ifdef USE_PARJAC
+  #include <omp.h>
+#endif
 
 int maxEventIterations = 20;
 double linearSparseSolverMaxDensity = 0.2;
-int linearSparseSolverMinSize = 4001;
+int linearSparseSolverMinSize = 201;
+double nonlinearSparseSolverMaxDensity = 0.2;
+int nonlinearSparseSolverMinSize = 10001;
+double maxStepFactor = 1e12;
 double newtonXTol = 1e-12;
 double newtonFTol = 1e-12;
+double steadyStateTol = 1e-3;
 const size_t SIZERINGBUFFER = 3;
 int compiledInDAEMode = 0;
+int compiledWithSymSolver = 0;
 double numericalDifferentiationDeltaXlinearize = 1e-8;
 double numericalDifferentiationDeltaXsolver = 1e-8;
+double homAdaptBend = 0.5;
+double homHEps = 1e-5;
+int homMaxLambdaSteps = 0;
+int homMaxNewtonSteps = 20;
+int homMaxTries = 10;
+double homTauDecreasingFactor = 10.0;
+double homTauDecreasingFactorPredictor = 2.0;
+double homTauIncreasingFactor = 2.0;
+double homTauIncreasingThreshold = 10.0;
+double homTauMax = 10.0;
+double homTauMin = 1e-4;
+double homTauStart = 0.2;
+int homBacktraceStrategy = 1;
 
 static double tolZC;
 
@@ -87,7 +111,7 @@ void updateDiscreteSystem(DATA *data, threadData_t *threadData)
   debugStreamPrint(LOG_EVENTS_V, 0, "updated discrete System");
 
   relationChanged = checkRelations(data);
-  discreteChanged = data->callback->checkForDiscreteChanges(data, threadData);
+  discreteChanged = checkForDiscreteChanges(data, threadData);
   while(discreteChanged || data->simulationInfo->needToIterate || relationChanged)
   {
     if(data->simulationInfo->needToIterate) {
@@ -114,7 +138,7 @@ void updateDiscreteSystem(DATA *data, threadData_t *threadData)
     }
 
     relationChanged = checkRelations(data);
-    discreteChanged = data->callback->checkForDiscreteChanges(data, threadData);
+    discreteChanged = checkForDiscreteChanges(data, threadData);
   }
   storeRelations(data);
 
@@ -350,14 +374,14 @@ void printParameters(DATA *data, int stream)
  *
  *  prints sparse structure of jacobian A
  *
- *  \param [in]  [data]
+ *  \param [in]  [sparsePattern]
+ *  \param [in]  [sizeRow]
+ *  \param [in]  [sizeCol]
  *  \param [in]  [stream]
  *
- *  \author lochel
  */
-void printSparseStructure(DATA *data, int stream)
+void printSparseStructure(SPARSE_PATTERN *sparsePattern, int sizeRows, int sizeCols, int stream, const char* name)
 {
-  const int index = data->callback->INDEX_JAC_A;
   unsigned int row, col, i, j;
   /* Will crash with a static size array */
   char *buffer = NULL;
@@ -365,29 +389,19 @@ void printSparseStructure(DATA *data, int stream)
   if (!ACTIVE_STREAM(stream))
     return;
 
-  buffer = (char*)omc_alloc_interface.malloc(sizeof(char)* 2*data->simulationInfo->analyticJacobians[index].sizeCols + 4);
+  buffer = (char*)omc_alloc_interface.malloc(sizeof(char)* 2*sizeCols + 4);
 
-  infoStreamPrint(stream, 1, "sparse structure of jacobian A [size: %ux%u]", data->simulationInfo->analyticJacobians[index].sizeRows, data->simulationInfo->analyticJacobians[index].sizeCols);
-  infoStreamPrint(stream, 0, "%u nonzero elements", data->simulationInfo->analyticJacobians[index].sparsePattern.numberOfNoneZeros);
-  /*
-  sprintf(buffer, "");
-  for(row=0; row < data->simulationInfo->analyticJacobians[index].sizeRows; row++)
-    sprintf(buffer, "%s%u ", buffer, data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[row]);
-  infoStreamPrint(stream, 0, "leadindex: %s", buffer);
+  infoStreamPrint(stream, 1, "sparse structure of %s [size: %ux%u]", name, sizeRows, sizeCols);
+  infoStreamPrint(stream, 0, "%u nonzero elements", sparsePattern->numberOfNoneZeros);
 
-  sprintf(buffer, "");
-  for(i=0; i < data->simulationInfo->analyticJacobians[index].sparsePattern.numberOfNoneZeros; i++)
-    sprintf(buffer, "%s%u ", buffer, data->simulationInfo->analyticJacobians[index].sparsePattern.index[i]);
-  infoStreamPrint(stream, 0, "index: %s", buffer);
-  */
   infoStreamPrint(stream, 1, "transposed sparse structure (rows: states)");
   i=0;
-  for(row=0; row < data->simulationInfo->analyticJacobians[index].sizeRows; row++)
+  for(row=0; row < sizeRows; row++)
   {
     j=0;
-    for(col=0; i < data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[row]; col++)
+    for(col=0; i < sparsePattern->leadindex[row+1]; col++)
     {
-      if(data->simulationInfo->analyticJacobians[index].sparsePattern.index[i] == col)
+      if(sparsePattern->index[i] == col)
       {
         buffer[j++] = '*';
         ++i;
@@ -885,14 +899,14 @@ void initializeDataStruc(DATA *data, threadData_t *threadData)
     tmpSimData.timeValue = 0;
     /* buffer for all variable values */
     tmpSimData.realVars = (modelica_real*) calloc(data->modelData->nVariablesReal, sizeof(modelica_real));
-    assertStreamPrint(threadData, 0 != tmpSimData.realVars, "out of memory");
+    assertStreamPrint(threadData, 0 == data->modelData->nVariablesReal || 0 != tmpSimData.realVars, "out of memory");
     tmpSimData.integerVars = (modelica_integer*) calloc(data->modelData->nVariablesInteger, sizeof(modelica_integer));
-    assertStreamPrint(threadData, 0 != tmpSimData.integerVars, "out of memory");
+    assertStreamPrint(threadData, 0 == data->modelData->nVariablesInteger || 0 != tmpSimData.integerVars, "out of memory");
     tmpSimData.booleanVars = (modelica_boolean*) calloc(data->modelData->nVariablesBoolean, sizeof(modelica_boolean));
-    assertStreamPrint(threadData, 0 != tmpSimData.booleanVars, "out of memory");
+    assertStreamPrint(threadData, 0 == data->modelData->nVariablesBoolean || 0 != tmpSimData.booleanVars, "out of memory");
 #if !defined(OMC_NVAR_STRING) || OMC_NVAR_STRING>0
     tmpSimData.stringVars = (modelica_string*) omc_alloc_interface.malloc_uncollectable(data->modelData->nVariablesString * sizeof(modelica_string));
-    assertStreamPrint(threadData, 0 != tmpSimData.stringVars, "out of memory");
+    assertStreamPrint(threadData, 0 == data->modelData->nVariablesString || 0 != tmpSimData.stringVars, "out of memory");
 #endif
     appendRingData(data->simulationData, &tmpSimData);
   }
@@ -928,14 +942,15 @@ void initializeDataStruc(DATA *data, threadData_t *threadData)
 
   /* set default solvers for algebraic loops */
 #if !defined(OMC_MINIMAL_RUNTIME)
-  data->simulationInfo->nlsMethod = NLS_HYBRID;
+  data->simulationInfo->nlsMethod = NLS_MIXED;
 #else
   data->simulationInfo->nlsMethod = NLS_HOMOTOPY;
 #endif
-  data->simulationInfo->lsMethod = LS_LAPACK;
-  data->simulationInfo->lssMethod = LS_UMFPACK;
+  data->simulationInfo->nlsLinearSolver = NLS_LS_DEFAULT;
+  data->simulationInfo->lsMethod = LS_DEFAULT;
+  data->simulationInfo->lssMethod = LSS_DEFAULT;
   data->simulationInfo->mixedMethod = MIXED_SEARCH;
-  data->simulationInfo->newtonStrategy = NEWTON_PURE;
+  data->simulationInfo->newtonStrategy = NEWTON_DAMPED2;
   data->simulationInfo->nlsCsvInfomation = 0;
   data->simulationInfo->currentContext = CONTEXT_ALGEBRAIC;
   data->simulationInfo->jacobianEvals = data->modelData->nStates;
@@ -974,6 +989,8 @@ void initializeDataStruc(DATA *data, threadData_t *threadData)
   /* buffer for inputs and outputs values */
   data->simulationInfo->inputVars = (modelica_real*) calloc(data->modelData->nInputVars, sizeof(modelica_real));
   data->simulationInfo->outputVars = (modelica_real*) calloc(data->modelData->nOutputVars, sizeof(modelica_real));
+  data->simulationInfo->setcVars = (modelica_real*) calloc(data->modelData->nSetcVars, sizeof(modelica_real));
+  data->simulationInfo->datainputVars = (modelica_real*) calloc(data->modelData->ndataReconVars, sizeof(modelica_real));
 
 #if !defined(OMC_NUM_MIXED_SYSTEMS) || OMC_NUM_MIXED_SYSTEMS>0
   /* buffer for mixed systems */
@@ -1020,6 +1037,11 @@ void initializeDataStruc(DATA *data, threadData_t *threadData)
   data->simulationInfo->daeModeData = (DAEMODE_DATA*) omc_alloc_interface.malloc_uncollectable(sizeof(DAEMODE_DATA));
   data->callback->initializeDAEmodeData(data, data->simulationInfo->daeModeData);
 
+  /* buffer for inline Data */
+  data->simulationInfo->inlineData = (INLINE_DATA*) omc_alloc_interface.malloc_uncollectable(sizeof(INLINE_DATA));
+  data->simulationInfo->inlineData->algVars = (modelica_real*) calloc(data->modelData->nStates, sizeof(modelica_real));
+  data->simulationInfo->inlineData->algOldVars = (modelica_real*) calloc(data->modelData->nStates, sizeof(modelica_real));
+
   /* buffer for analytical jacobians */
   data->simulationInfo->analyticJacobians = (ANALYTIC_JACOBIAN*) omc_alloc_interface.malloc_uncollectable(data->modelData->nJacobians*sizeof(ANALYTIC_JACOBIAN));
 
@@ -1030,7 +1052,7 @@ void initializeDataStruc(DATA *data, threadData_t *threadData)
   data->simulationInfo->extObjs = NULL;
   data->simulationInfo->extObjs = (void**) calloc(data->modelData->nExtObjs, sizeof(void*));
 
-  assertStreamPrint(threadData, 0 != data->simulationInfo->extObjs, "error allocating external objects");
+  assertStreamPrint(threadData, 0 == data->modelData->nExtObjs || 0 != data->simulationInfo->extObjs, "error allocating external objects");
 
 #if !defined(OMC_MINIMAL_LOGGING)
   /* initial chattering info */
@@ -1068,7 +1090,7 @@ void initializeDataStruc(DATA *data, threadData_t *threadData)
   /* initial delay */
 #if !defined(OMC_NDELAY_EXPRESSIONS) || OMC_NDELAY_EXPRESSIONS>0
   data->simulationInfo->delayStructure = (RINGBUFFER**)malloc(data->modelData->nDelayExpressions * sizeof(RINGBUFFER*));
-  assertStreamPrint(threadData, 0 != data->simulationInfo->delayStructure, "out of memory");
+  assertStreamPrint(threadData, 0 == data->modelData->nDelayExpressions || 0 != data->simulationInfo->delayStructure, "out of memory");
 
   for(i=0; i<data->modelData->nDelayExpressions; i++)
     data->simulationInfo->delayStructure[i] = allocRingBuffer(1024, sizeof(TIME_AND_VALUE));
@@ -1174,11 +1196,6 @@ void deInitializeDataStruc(DATA *data)
   free(data->simulationInfo->booleanParameter);
   omc_alloc_interface.free_uncollectable(data->simulationInfo->stringParameter);
 
-  if (data->modelData->nStateSets) {
-    /* free buffer for state sets */
-    omc_alloc_interface.free_uncollectable(data->simulationInfo->stateSetData);
-  }
-
   if (data->modelData->nMixedSystems) {
     /* free buffer of mixed systems */
     omc_alloc_interface.free_uncollectable(data->simulationInfo->mixedSystemData);
@@ -1204,6 +1221,8 @@ void deInitializeDataStruc(DATA *data)
   /* free inputs and output */
   free(data->simulationInfo->inputVars);
   free(data->simulationInfo->outputVars);
+  free(data->simulationInfo->setcVars);
+  free(data->simulationInfo->datainputVars);
 
   /* free external objects buffer */
   free(data->simulationInfo->extObjs);
@@ -1222,6 +1241,10 @@ void deInitializeDataStruc(DATA *data)
   /* free stateset data */
   freeStateSetData(data);
 #endif
+  if (data->modelData->nStateSets) {
+    /* free buffer for state sets */
+    omc_alloc_interface.free_uncollectable(data->simulationInfo->stateSetData);
+  }
 
   /* free parameter sensitivities */
   if (omc_flag[FLAG_IDAS])
@@ -1461,6 +1484,7 @@ const char *context_string[CONTEXT_MAX] = {
  "context algebraic evaluation",
  "context event search",
  "context jacobian evaluation",
+ "context symbolica jacobian evaluation"
 };
 
 /*! \fn setContext
@@ -1474,8 +1498,10 @@ const char *context_string[CONTEXT_MAX] = {
 void setContext(DATA* data, double* currentTime, int currentContext){
   data->simulationInfo->currentContextOld =  data->simulationInfo->currentContext;
   data->simulationInfo->currentContext =  currentContext;
-  infoStreamPrint(LOG_SOLVER_CONTEXT, 0, "+++ Set context %s +++ at time %f", context_string[data->simulationInfo->currentContext], *currentTime);
-  if (currentContext == CONTEXT_JACOBIAN){
+  infoStreamPrint(LOG_SOLVER_CONTEXT, 0, "+++ Set context %s +++ at time %f", context_string[currentContext], *currentTime);
+  if (currentContext == CONTEXT_JACOBIAN ||
+      currentContext == CONTEXT_SYM_JACOBIAN)
+  {
     data->simulationInfo->currentJacobianEval = 0;
   }
 }
@@ -1487,9 +1513,12 @@ void setContext(DATA* data, double* currentTime, int currentContext){
  * Increase Jacobian column context in simulation info object
  */
 void increaseJacContext(DATA* data){
-  if (data->simulationInfo->currentContext == CONTEXT_JACOBIAN){
+  int currentContext = data->simulationInfo->currentContext;
+  if (currentContext == CONTEXT_JACOBIAN ||
+      currentContext == CONTEXT_SYM_JACOBIAN)
+  {
     data->simulationInfo->currentJacobianEval++;
-    infoStreamPrint(LOG_SOLVER_CONTEXT, 0, "+++ Increase Jacobian column context %s +++ to %d", context_string[data->simulationInfo->currentContext], data->simulationInfo->currentJacobianEval);
+    infoStreamPrint(LOG_SOLVER_CONTEXT, 0, "+++ Increase Jacobian column context %s +++ to %d", context_string[currentContext], data->simulationInfo->currentJacobianEval);
   }
 }
 
