@@ -41,6 +41,7 @@
 #include <QLineEdit>
 #include <QCheckBox>
 #include <QToolButton>
+#include <QStandardItemModel>
 
 class ModelWidget;
 class InfoBar;
@@ -102,6 +103,9 @@ public:
     , mFolded(false)
     , mFoldingEndIncluded(false)
     , mFoldingState(false)
+    , mFoldingEndState(false)
+    , mFoldingEnd(false)
+    , mFoldingStartIndex(-1)
     , mLeadingSpaces(-1)
   {}
   ~TextBlockUserData();
@@ -143,6 +147,12 @@ public:
   inline bool foldingEndIncluded() const {return mFoldingEndIncluded;}
   inline void setFoldingState(bool foldingState) {mFoldingState = foldingState;}
   inline bool foldingState() const {return mFoldingState;}
+  inline void setFoldingEndState(bool foldingEndState) {mFoldingEndState = foldingEndState;}
+  inline bool foldingEndState() const {return mFoldingEndState;}
+  inline void setFoldingEnd(bool foldingEnd) {mFoldingEnd = foldingEnd;}
+  inline bool foldingEnd() const {return mFoldingEnd;}
+  inline void setFoldingStartIndex(int foldingStartIndex) {mFoldingStartIndex = foldingStartIndex;}
+  inline int foldingStartIndex() const {return mFoldingStartIndex;}
 
   inline void setLeadingSpaces(int leadingSpaces) {mLeadingSpaces = leadingSpaces;}
   inline int getLeadingSpaces() {return mLeadingSpaces;}
@@ -153,6 +163,9 @@ private:
   bool mFolded;
   bool mFoldingEndIncluded;
   bool mFoldingState;
+  bool mFoldingEndState;
+  bool mFoldingEnd;
+  int mFoldingStartIndex;
   int mLeadingSpaces;
 };
 
@@ -197,96 +210,159 @@ public:
   bool mHasBreakpoint;
 };
 
+class CompleterItem
+{
+public:
+  CompleterItem() {}
+  CompleterItem(const QString &key, const QString &value, const QString &select);
+  CompleterItem(const QString &key, const QString &value, const QString &select, const QString &description);
+  CompleterItem(const QString &value, const QString &description);
+  QString mKey;
+  QString mValue;
+  QString mSelect;
+  QString mDescription;
+
+  // Supposing two items with equal keys and different descriptions.
+  bool operator<(const CompleterItem &other) const
+  {
+    return (mKey < other.mKey) || ((mKey == other.mKey) && (mDescription < other.mDescription));
+  }
+  bool operator==(const CompleterItem &other) const
+  {
+    return mKey == other.mKey && mDescription == other.mDescription;
+  }
+};
+
+Q_DECLARE_METATYPE(CompleterItem)
+
+class BaseEditor;
+class QCompleter;
+class PlainTextEdit : public QPlainTextEdit
+{
+  Q_OBJECT
+public:
+  PlainTextEdit(BaseEditor *pBaseEditor);
+  bool eventFilter(QObject *pObject, QEvent *pEvent) override;
+  LineNumberArea* getLineNumberArea() {return mpLineNumberArea;}
+  void clearCompleter();
+  void insertCompleterSymbols(QList<CompleterItem> symbols, const QString &iconResource);
+  void insertCompleterKeywords(QStringList keywords);
+  void insertCompleterTypes(QStringList types);
+  void insertCompleterCodeSnippets(QList<CompleterItem> items);
+  void setCanHaveBreakpoints(bool canHaveBreakpoints);
+  bool canHaveBreakpoints() {return mCanHaveBreakpoints;}
+  int lineNumberAreaWidth();
+  void lineNumberAreaPaintEvent(QPaintEvent *event);
+  void lineNumberAreaMouseEvent(QMouseEvent *event);
+  void goToLineNumber(int lineNumber);
+  QCompleter *completer();
+  bool isUndoAvailable() {return mIsUndoAvailable;}
+  bool isRedoAvailable() {return mIsRedoAvailable;}
+  void setCompletionCharacters(QString chars) { mCompletionCharacters = chars; }
+private:
+  BaseEditor *mpBaseEditor;
+  LineNumberArea *mpLineNumberArea;
+  bool mCanHaveBreakpoints;
+  QTextCharFormat mParenthesesMatchFormat;
+  QTextCharFormat mParenthesesMisMatchFormat;
+  QWidget *mpCompleterToolTipWidget;
+  Label *mpCompleterToolTipLabel;
+  QStandardItemModel* mpStandardItemModel;
+  QCompleter *mpCompleter;
+  bool mIsUndoAvailable;
+  bool mIsRedoAvailable;
+  QString mCompletionCharacters;
+
+  void highlightCurrentLine();
+  void highlightParentheses();
+  void setLineWrapping();
+  QString plainTextFromSelection(const QTextCursor &cursor) const;
+  static QString convertToPlainText(const QString &txt);
+  void moveCursorVisible(bool ensureVisible = true);
+  void ensureCursorVisible();
+  void toggleBreakpoint(const QString fileName, int lineNumber);
+  void indentOrUnindent(bool doIndent);
+  void foldOrUnfold(bool unFold);
+  void handleHomeKey(bool keepAnchor);
+  void toggleBlockVisible(const QTextBlock &block);
+private slots:
+  void showCompletionItemToolTip(const QModelIndex & index);
+  void insertCompletionItem(const QModelIndex & index);
+public slots:
+  void updateLineNumberAreaWidth(int newBlockCount);
+  void updateLineNumberArea(const QRect &rect, int dy);
+  void updateHighlights();
+  void updateCursorPosition();
+  void textSettingsChanged();
+  void setUndoAvailable(bool available) {mIsUndoAvailable = available;}
+  void setRedoAvailable(bool available) {mIsRedoAvailable = available;}
+  void showTabsAndSpaces(bool On);
+  void toggleBreakpoint();
+  void foldAll();
+  void unFoldAll();
+  void resetZoom();
+  void zoomIn();
+  void zoomOut();
+protected:
+  virtual void resizeEvent(QResizeEvent *pEvent) override;
+  virtual void keyPressEvent(QKeyEvent *pEvent) override;
+  virtual QMimeData* createMimeDataFromSelection() const override;
+  virtual bool canInsertFromMimeData(const QMimeData *source) const override;
+  virtual void insertFromMimeData(const QMimeData *source) override;
+  virtual void focusInEvent(QFocusEvent *event) override;
+  virtual void focusOutEvent(QFocusEvent *event) override;
+  void paintEvent(QPaintEvent *e) override;
+  void wheelEvent(QWheelEvent *event) override;
+};
+
 class BaseEditor : public QWidget
 {
   Q_OBJECT
-private:
-  class PlainTextEdit : public QPlainTextEdit
-  {
-  public:
-    PlainTextEdit(BaseEditor *pBaseEditor);
-    LineNumberArea* getLineNumberArea() {return mpLineNumberArea;}
-    int lineNumberAreaWidth();
-    void lineNumberAreaPaintEvent(QPaintEvent *event);
-    void lineNumberAreaMouseEvent(QMouseEvent *event);
-    void goToLineNumber(int lineNumber);
-    void updateLineNumberAreaWidth(int newBlockCount);
-    void updateLineNumberArea(const QRect &rect, int dy);
-    void updateHighlights();
-    void updateCursorPosition();
-    void setLineWrapping();
-    void toggleBreakpoint(const QString fileName, int lineNumber);
-    void indentOrUnindent(bool doIndent);
-    void moveCursorVisible(bool ensureVisible = true);
-    void ensureCursorVisible();
-  private:
-    BaseEditor *mpBaseEditor;
-    LineNumberArea *mpLineNumberArea;
-    QTextCharFormat mParenthesesMatchFormat;
-    QTextCharFormat mParenthesesMisMatchFormat;
-
-    void highlightCurrentLine();
-    void highlightParentheses();
-    QString plainTextFromSelection(const QTextCursor &cursor) const;
-    static QString convertToPlainText(const QString &txt);
-  protected:
-    virtual void resizeEvent(QResizeEvent *pEvent);
-    virtual void keyPressEvent(QKeyEvent *pEvent);
-    virtual QMimeData* createMimeDataFromSelection() const;
-    virtual void focusInEvent(QFocusEvent *event);
-    virtual void focusOutEvent(QFocusEvent *event);
-    void paintEvent(QPaintEvent *e);
-  };
 public:
   BaseEditor(QWidget *pParent);
   ModelWidget *getModelWidget() {return mpModelWidget;}
   InfoBar* getInfoBar() {return mpInfoBar;}
   PlainTextEdit *getPlainTextEdit() {return mpPlainTextEdit;}
   FindReplaceWidget* getFindReplaceWidget() {return mpFindReplaceWidget;}
-  void setCanHaveBreakpoints(bool canHaveBreakpoints);
-  bool canHaveBreakpoints() {return mCanHaveBreakpoints;}
-  QAction *getToggleBreakpointAction() {return mpToggleBreakpointAction;}
+  QAction* getToggleBreakpointAction() {return mpToggleBreakpointAction;}
+  QAction* getFoldAllAction() {return mpFoldAllAction;}
+  QAction* getUnFoldAllAction() {return mpUnFoldAllAction;}
   DocumentMarker* getDocumentMarker() {return mpDocumentMarker;}
-  void goToLineNumber(int lineNumber);
-  void toggleBlockVisible(const QTextBlock &block);
+  void setForceSetPlainText(bool forceSetPlainText) {mForceSetPlainText = forceSetPlainText;}
+  virtual void popUpCompleter () = 0;
+  virtual QString wordUnderCursor();
+  bool isModelicaModelInPackageOneFile();
 private:
   void initialize();
   void createActions();
-  void foldOrUnfold(bool unFold);
 protected:
   ModelWidget *mpModelWidget;
   InfoBar *mpInfoBar;
   PlainTextEdit *mpPlainTextEdit;
   FindReplaceWidget *mpFindReplaceWidget;
-  bool mCanHaveBreakpoints;
   QAction *mpFindReplaceAction;
   QAction *mpClearFindReplaceTextsAction;
   QAction *mpGotoLineNumberAction;
   QAction *mpShowTabsAndSpacesAction;
   QAction *mpToggleBreakpointAction;
+  QAction *mpResetZoomAction;
+  QAction *mpZoomInAction;
+  QAction *mpZoomOutAction;
   QAction *mpToggleCommentSelectionAction;
   QAction *mpFoldAllAction;
   QAction *mpUnFoldAllAction;
   DocumentMarker *mpDocumentMarker;
+  bool mForceSetPlainText;
 
   QMenu* createStandardContextMenu();
 private slots:
   virtual void showContextMenu(QPoint point) = 0;
 public slots:
-  void textSettingsChanged();
-  void updateLineNumberAreaWidth(int newBlockCount);
-  void updateLineNumberArea(const QRect &rect, int dy);
-  void updateHighlights();
-  void updateCursorPosition();
   virtual void contentsHasChanged(int position, int charsRemoved, int charsAdded) = 0;
   void showFindReplaceWidget();
   void clearFindReplaceTexts();
   void showGotoLineNumberDialog();
-  void showTabsAndSpaces(bool On);
-  void toggleBreakpoint();
   virtual void toggleCommentSelection();
-  void foldAll();
-  void unFoldAll();
 };
 
 class LineNumberArea : public QWidget
@@ -297,20 +373,20 @@ public:
   {
     mpBaseEditor = pBaseEditor;
   }
-  QSize sizeHint() const
+  QSize sizeHint() const override
   {
     return QSize(mpBaseEditor->getPlainTextEdit()->lineNumberAreaWidth(), 0);
   }
 protected:
-  virtual void paintEvent(QPaintEvent *event)
+  virtual void paintEvent(QPaintEvent *event) override
   {
     mpBaseEditor->getPlainTextEdit()->lineNumberAreaPaintEvent(event);
   }
-  virtual void mouseMoveEvent(QMouseEvent *event)
+  virtual void mouseMoveEvent(QMouseEvent *event) override
   {
     mpBaseEditor->getPlainTextEdit()->lineNumberAreaMouseEvent(event);
   }
-  virtual void mousePressEvent(QMouseEvent *event)
+  virtual void mousePressEvent(QMouseEvent *event) override
   {
     mpBaseEditor->getPlainTextEdit()->lineNumberAreaMouseEvent(event);
   }
@@ -350,7 +426,7 @@ public slots:
   void replace();
   void replaceAll();
 protected:
-  virtual void keyPressEvent(QKeyEvent *pEvent);
+  virtual void keyPressEvent(QKeyEvent *pEvent) override;
 protected slots:
   void validateRegularExpression(const QString &text);
   void regularExpressionSelected(bool selected);
