@@ -44,6 +44,7 @@
 #include <QTreeView>
 #include <QSortFilterProxyModel>
 
+class CompleterItem;
 class GraphicsView;
 class ModelWidget;
 class ShapeAnnotation;
@@ -87,12 +88,13 @@ public:
   void setLibraryType(LibraryType libraryType) {mLibraryType = libraryType;}
   void setSystemLibrary(bool systemLibrary) {mSystemLibrary = systemLibrary;}
   bool isSystemLibrary() {return mSystemLibrary;}
-  void setModelWidget(ModelWidget *pModelWidget) {mpModelWidget = pModelWidget;}
+  void setModelWidget(ModelWidget *pModelWidget);
   ModelWidget* getModelWidget() {return mpModelWidget;}
   void setName(QString name) {mName = name;}
   const QString& getName() const {return mName;}
   void setNameStructure(QString nameStructure) {mNameStructure = nameStructure;}
   const QString& getNameStructure() {return mNameStructure;}
+  QString getWhereToMoveFMU();
   void setClassInformation(OMCInterface::getClassInformation_res classInformation);
   void setFileName(QString fileName) {mFileName = fileName;}
   const QString& getFileName() const {return mFileName;}
@@ -116,7 +118,7 @@ public:
   QPixmap getDragPixmap() {return mDragPixmap;}
   void setClassTextBefore(QString classTextBefore) {mClassTextBefore = classTextBefore;}
   QString getClassTextBefore() {return mClassTextBefore;}
-  void setClassText(QString classText) {mClassText = classText;}
+  void setClassText(QString classText);
   QString getClassText(LibraryTreeModel *pLibraryTreeModel);
   void setClassTextAfter(QString classTextAfter) {mClassTextAfter = classTextAfter;}
   QString getClassTextAfter() {return mClassTextAfter;}
@@ -124,11 +126,14 @@ public:
   bool isExpanded() const {return mExpanded;}
   void setNonExisting(bool nonExisting) {mNonExisting = nonExisting;}
   bool isNonExisting() const {return mNonExisting;}
+  bool isAccessAnnotationsEnabled() const {return mAccessAnnotations;}
+  void setAccessAnnotations(bool accessAnnotations) {mAccessAnnotations = accessAnnotations;}
   void setOMSElement(oms_element_t *pOMSComponent) {mpOMSElement = pOMSComponent;}
   oms_element_t* getOMSElement() const {return mpOMSElement;}
   bool isSystemElement() const {return (mpOMSElement && (mpOMSElement->type == oms_element_system));}
   bool isComponentElement() const {return (mpOMSElement && (mpOMSElement->type == oms_element_component));}
   bool isFMUComponent() const {return (mpOMSElement && (mpOMSElement->type == oms_element_component) && (mComponentType == oms_component_fmu));}
+  bool isExternalTLMModelComponent() const {return (mpOMSElement && (mpOMSElement->type == oms_element_component) && (mComponentType == oms_component_external));}
   bool isTableComponent() const {return (mpOMSElement && (mpOMSElement->type == oms_element_component) && (mComponentType == oms_component_table));}
   void setSystemType(oms_system_enu_t type) {mSystemType = type;}
   oms_system_enu_t getSystemType() {return mSystemType;}
@@ -146,6 +151,8 @@ public:
   oms_tlmbusconnector_t* getOMSTLMBusConnector() const {return mpOMSTLMBusConnector;}
   void setFMUInfo(const oms_fmu_info_t *pFMUInfo) {mpFMUInfo = pFMUInfo;}
   const oms_fmu_info_t* getFMUInfo() const {return mpFMUInfo;}
+  void setExternalTLMModelInfo(const oms_external_tlm_model_info_t *pExternalTLMModelInfo) { mpExternalTLMModelInfo = pExternalTLMModelInfo;}
+  const oms_external_tlm_model_info_t* getExternalTLMModelInfo() const {return mpExternalTLMModelInfo;}
   void setSubModelPath(QString subModelPath) {mSubModelPath = subModelPath;}
   QString getSubModelPath() const {return mSubModelPath;}
   oms_modelState_enu_t getModelState() const {return mModelState;}
@@ -161,6 +168,10 @@ public:
   void addInheritedClass(LibraryTreeItem *pLibraryTreeItem);
   void removeInheritedClasses();
   QList<LibraryTreeItem*> getInheritedClasses() const {return mInheritedClasses;}
+  QList<LibraryTreeItem*> getInheritedClassesDeepList();
+  LibraryTreeItem *getDirectComponentsClass(const QString &name);
+  LibraryTreeItem *getComponentsClass(const QString &name);
+  void tryToComplete(QList<CompleterItem> &completionClasses, QList<CompleterItem> &completionComponents, const QString &lastPart);
   void removeChild(LibraryTreeItem *pLibraryTreeItem);
   QVariant data(int column, int role = Qt::DisplayRole) const;
   int row() const;
@@ -178,15 +189,19 @@ public:
   void emitConnectionAdded(LineAnnotation *pConnectionLineAnnotation) {emit connectionAdded(pConnectionLineAnnotation);}
   void emitCoOrdinateSystemUpdated(GraphicsView *pGraphicsView) {emit coOrdinateSystemUpdated(pGraphicsView);}
   bool isInstantiated();
+  QString getHTMLDescription() const;
 
   OMCInterface::getClassInformation_res mClassInformation;
   SimulationOptions mSimulationOptions;
   OMSSimulationOptions mOMSSimulationOptions;
+  const QList<ComponentInfo *> &getComponentsList();
 private:
   bool mIsRootItem;
   LibraryTreeItem *mpParentLibraryTreeItem;
   QList<LibraryTreeItem*> mChildren;
   QList<LibraryTreeItem*> mInheritedClasses;
+  QList<ComponentInfo*> mComponents;
+  bool mComponentsLoaded;
   LibraryType mLibraryType;
   bool mSystemLibrary;
   ModelWidget *mpModelWidget;
@@ -204,6 +219,7 @@ private:
   QString mClassTextAfter;
   bool mExpanded;
   bool mNonExisting;
+  bool mAccessAnnotations;
   oms_element_t *mpOMSElement;
   oms_system_enu_t mSystemType;
   oms_component_enu_t mComponentType;
@@ -211,6 +227,7 @@ private:
   oms_busconnector_t *mpOMSBusConnector;
   oms_tlmbusconnector_t *mpOMSTLMBusConnector;
   const oms_fmu_info_t *mpFMUInfo;
+  const oms_external_tlm_model_info_t *mpExternalTLMModelInfo;
   QString mSubModelPath;
   oms_modelState_enu_t mModelState;
 signals:
@@ -246,7 +263,7 @@ private:
   LibraryWidget *mpLibraryWidget;
   bool mShowOnlyModelica;
 protected:
-  virtual bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const;
+  virtual bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const override;
 };
 
 class LibraryTreeModel : public QAbstractItemModel
@@ -255,13 +272,13 @@ class LibraryTreeModel : public QAbstractItemModel
 public:
   LibraryTreeModel(LibraryWidget *pLibraryWidget);
   LibraryTreeItem* getRootLibraryTreeItem() {return mpRootLibraryTreeItem;}
-  int columnCount(const QModelIndex &parent = QModelIndex()) const;
-  int rowCount(const QModelIndex &parent = QModelIndex()) const;
-  QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
-  QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;
-  QModelIndex parent(const QModelIndex & index) const;
-  QVariant data(const QModelIndex & index, int role = Qt::DisplayRole) const;
-  Qt::ItemFlags flags(const QModelIndex &index) const;
+  int columnCount(const QModelIndex &parent = QModelIndex()) const override;
+  int rowCount(const QModelIndex &parent = QModelIndex()) const override;
+  QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const override;
+  QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const override;
+  QModelIndex parent(const QModelIndex & index) const override;
+  QVariant data(const QModelIndex & index, int role = Qt::DisplayRole) const override;
+  Qt::ItemFlags flags(const QModelIndex &index) const override;
   LibraryTreeItem* findLibraryTreeItem(const QString &name, LibraryTreeItem *pLibraryTreeItem = 0,
                                        Qt::CaseSensitivity caseSensitivity = Qt::CaseSensitive) const;
   LibraryTreeItem* findLibraryTreeItem(const QRegExp &regExp, LibraryTreeItem *pLibraryTreeItem = 0) const;
@@ -271,7 +288,7 @@ public:
   QModelIndex libraryTreeItemIndex(const LibraryTreeItem *pLibraryTreeItem) const;
   void addModelicaLibraries();
   LibraryTreeItem* createLibraryTreeItem(QString name, LibraryTreeItem *pParentLibraryTreeItem, bool isSaved = true,
-                                         bool isSystemLibrary = false, bool load = false, int row = -1);
+                                         bool isSystemLibrary = false, bool load = false, int row = -1, bool activateAccessAnnotations = false);
   LibraryTreeItem* createNonExistingLibraryTreeItem(QString nameStructure);
   void createLibraryTreeItems(QFileInfo fileInfo, LibraryTreeItem *pParentLibraryTreeItem);
   LibraryTreeItem* createLibraryTreeItem(LibraryTreeItem::LibraryType type, QString name, QString nameStructure, QString path, bool isSaved,
@@ -286,12 +303,13 @@ public:
   void updateLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem);
   void updateLibraryTreeItemClassText(LibraryTreeItem *pLibraryTreeItem);
   void updateLibraryTreeItemClassTextManually(LibraryTreeItem *pLibraryTreeItem, QString contents);
+  void updateChildLibraryTreeItemClassText(LibraryTreeItem *pLibraryTreeItem, QString contents, QString fileName);
   void readLibraryTreeItemClassText(LibraryTreeItem *pLibraryTreeItem);
   LibraryTreeItem* getContainingFileParentLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem);
   void loadLibraryTreeItemPixmap(LibraryTreeItem *pLibraryTreeItem);
   void loadDependentLibraries(QStringList libraries);
   LibraryTreeItem* getLibraryTreeItemFromFile(QString fileName, int lineNumber);
-  void showModelWidget(LibraryTreeItem *pLibraryTreeItem, bool show = true, StringHandler::ViewType viewType = StringHandler::NoView);
+  void showModelWidget(LibraryTreeItem *pLibraryTreeItem, bool show = true);
   void showHideProtectedClasses();
   bool unloadClass(LibraryTreeItem *pLibraryTreeItem, bool askQuestion = true);
   bool unloadCompositeModelOrTextFile(LibraryTreeItem *pLibraryTreeItem, bool askQuestion = true);
@@ -312,7 +330,6 @@ private:
   QModelIndex libraryTreeItemIndexHelper(const LibraryTreeItem *pLibraryTreeItem, const LibraryTreeItem *pParentLibraryTreeItem,
                                          const QModelIndex &parentIndex) const;
   LibraryTreeItem* getLibraryTreeItemFromFileHelper(LibraryTreeItem *pLibraryTreeItem, QString fileName, int lineNumber);
-  void updateChildLibraryTreeItemClassText(LibraryTreeItem *pLibraryTreeItem, QString contents, QString fileName);
   void updateOMSLibraryTreeItemClassText(LibraryTreeItem *pLibraryTreeItem);
   void readLibraryTreeItemClassTextFromText(LibraryTreeItem *pLibraryTreeItem, QString contents);
   QString readLibraryTreeItemClassTextFromFile(LibraryTreeItem *pLibraryTreeItem);
@@ -321,7 +338,7 @@ public:
   void updateOMSChildLibraryTreeItemClassText(LibraryTreeItem *pLibraryTreeItem);
 private:
   LibraryTreeItem* createLibraryTreeItemImpl(QString name, LibraryTreeItem *pParentLibraryTreeItem, bool isSaved = true,
-                                             bool isSystemLibrary = false, bool load = false, int row = -1);
+                                             bool isSystemLibrary = false, bool load = false, int row = -1, bool activateAccessAnnotations = false);
   void createNonExistingLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem, LibraryTreeItem *pParentLibraryTreeItem, bool isSaved = true,
                                         int row = -1);
   void createLibraryTreeItemsImpl(QFileInfo fileInfo, LibraryTreeItem *pParentLibraryTreeItem);
@@ -343,7 +360,7 @@ private:
   void deleteFileHelper(LibraryTreeItem *pLibraryTreeItem, LibraryTreeItem *pParentLibraryTreeItem);
   void deleteFileChildren(LibraryTreeItem *pLibraryTreeItem);
 protected:
-  Qt::DropActions supportedDropActions() const;
+  Qt::DropActions supportedDropActions() const override;
 };
 
 class LibraryTreeView : public QTreeView
@@ -355,10 +372,6 @@ public:
 private:
   LibraryWidget *mpLibraryWidget;
   QAction *mpOpenClassAction;
-  QAction *mpViewIconAction;
-  QAction *mpViewDiagramAction;
-  QAction *mpViewTextAction;
-  QAction *mpViewDocumentationAction;
   QAction *mpInformationAction;
   QAction *mpNewModelicaClassAction;
   QAction *mpNewModelicaClassEmptyAction;
@@ -391,6 +404,7 @@ private:
   QAction *mpRenameAction;
   QAction *mpDeleteAction;
   QAction *mpExportFMUAction;
+  QAction *mpExportReadonlyPackageAction;
   QAction *mpExportEncryptedPackageAction;
   QAction *mpExportXMLAction;
   QAction *mpExportFigaroAction;
@@ -405,13 +419,10 @@ private:
   LibraryTreeItem* getSelectedLibraryTreeItem();
   void libraryTreeItemExpanded(LibraryTreeItem* pLibraryTreeItem);
 public slots:
-  void libraryTreeItemExpanded(QModelIndex index);
+  void libraryTreeItemExpanded(const QModelIndex &index);
+  void libraryTreeItemDoubleClicked(const QModelIndex &index);
   void showContextMenu(QPoint point);
   void openClass();
-  void viewIcon();
-  void viewDiagram();
-  void viewText();
-  void viewDocumentation();
   void openInformationDialog();
   void createNewModelicaClass();
   void createNewModelicaClassEmpty();
@@ -442,6 +453,7 @@ public slots:
   void deleteTextFile();
   void exportModelFMU();
   void exportEncryptedPackage();
+  void exportReadonlyPackage();
   void exportModelXML();
   void exportModelFigaro();
   void updateBindings();
@@ -452,9 +464,8 @@ public slots:
   void OMSRename();
   void unloadOMSModel();
 protected:
-  virtual void mouseDoubleClickEvent(QMouseEvent *event);
-  virtual void startDrag(Qt::DropActions supportedActions);
-  virtual void keyPressEvent(QKeyEvent *event);
+  virtual void startDrag(Qt::DropActions supportedActions) override;
+  virtual void keyPressEvent(QKeyEvent *event) override;
 };
 
 class LibraryWidget : public QWidget
@@ -485,6 +496,7 @@ private:
   LibraryTreeModel *mpLibraryTreeModel;
   LibraryTreeProxyModel *mpLibraryTreeProxyModel;
   LibraryTreeView *mpLibraryTreeView;
+  bool multipleTopLevelClasses(const QStringList &classesList, const QString &fileName);
   bool saveModelicaLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem);
   bool saveModelicaLibraryTreeItemHelper(LibraryTreeItem *pLibraryTreeItem);
   bool saveModelicaLibraryTreeItemOneFile(LibraryTreeItem *pLibraryTreeItem);
@@ -496,6 +508,7 @@ private:
   void saveOMSLibraryTreeItemHelper(LibraryTreeItem *pLibraryTreeItem, QString fileName);
   bool saveCompositeModelLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem);
   bool saveAsCompositeModelLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem);
+  bool saveAsOMSLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem);
   bool saveCompositeModelLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem, QString fileName);
   bool saveTotalLibraryTreeItemHelper(LibraryTreeItem *pLibraryTreeItem);
 public slots:
