@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -29,20 +29,34 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- * RCS: $Id$
- *
  */
 
 #ifndef SIMULATIONDIALOG_H
 #define SIMULATIONDIALOG_H
 
-#include "MainWindow.h"
+#include "Util/Helper.h"
 #include "SimulationOptions.h"
+#include "OpcUaClient.h"
 
+#include <QDialog>
+#include <QTreeWidget>
+#include <QScrollArea>
+#include <QGroupBox>
+#include <QRadioButton>
+#include <QSpinBox>
+#include <QComboBox>
+#include <QToolButton>
+#include <QCheckBox>
+#include <QPushButton>
+#include <QDialogButtonBox>
+#include <QGridLayout>
+#include <QDateTime>
+
+class Label;
 class SimulationOutputWidget;
+class LibraryTreeItem;
+class TranslationFlagsWidget;
 
 class ArchivedSimulationItem : public QTreeWidgetItem
 {
@@ -69,46 +83,52 @@ private:
   SimulationOutputWidget *mpSimulationOutputWidget;
 };
 
-class MainWindow;
 class SimulationDialog : public QDialog
 {
   Q_OBJECT
 public:
-  SimulationDialog(MainWindow *pParent = 0);
+  SimulationDialog(QWidget *pParent = 0);
   ~SimulationDialog();
   QTreeWidget* getArchivedSimulationsTreeWidget() {return mpArchivedSimulationsTreeWidget;}
-  void show(LibraryTreeNode *pLibraryTreeNode, bool isReSimulate, SimulationOptions simulationOptions);
-  void directSimulate(LibraryTreeNode *pLibraryTreeNode, bool launchTransformationalDebugger, bool launchAlgorithmicDebugger);
+  void show(LibraryTreeItem *pLibraryTreeItem, bool isReSimulate, SimulationOptions simulationOptions);
+  void directSimulate(LibraryTreeItem *pLibraryTreeItem, bool launchTransformationalDebugger, bool launchAlgorithmicDebugger,
+                      bool launchAnimation);
+  QList<SimulationOutputWidget*> getSimulationOutputWidgetsList() {return mSimulationOutputWidgetsList;}
+  OpcUaClient* getOpcUaClient(int port);
+  void removeSimulationOutputWidget(SimulationOutputWidget* pSimulationOutputWidget);
 private:
-  MainWindow *mpMainWindow;
   Label *mpSimulationHeading;
   QFrame *mpHorizontalLine;
   QTabWidget *mpSimulationTabWidget;
   // General Tab
   QWidget *mpGeneralTab;
+  QScrollArea *mpGeneralTabScrollArea;
   QGroupBox *mpSimulationIntervalGroupBox;
   Label *mpStartTimeLabel;
   QLineEdit *mpStartTimeTextBox;
   Label *mpStopTimeLabel;
   QLineEdit *mpStopTimeTextBox;
+  QRadioButton *mpNumberofIntervalsRadioButton;
+  QSpinBox *mpNumberofIntervalsSpinBox;
+  QRadioButton *mpIntervalRadioButton;
+  QLineEdit *mpIntervalTextBox;
   QGroupBox *mpIntegrationGroupBox;
   Label *mpMethodLabel;
   QComboBox *mpMethodComboBox;
   QToolButton *mpMehtodHelpButton;
   Label *mpToleranceLabel;
   QLineEdit *mpToleranceTextBox;
-  QGroupBox *mpDasslOptionsGroupBox;
-  Label *mpDasslJacobianLabel;
-  QComboBox *mpDasslJacobianComboBox;
-  QCheckBox *mpDasslRootFindingCheckBox;
-  QCheckBox *mpDasslRestartCheckBox;
-  Label *mpDasslInitialStepSizeLabel;
-  QLineEdit *mpDasslInitialStepSizeTextBox;
-  Label *mpDasslMaxStepSizeLabel;
-  QLineEdit *mpDasslMaxStepSizeTextBox;
-  Label *mpDasslMaxIntegrationOrderLabel;
-  QSpinBox *mpDasslMaxIntegrationOrderSpinBox;
-  QCheckBox *mpSaveSimulationCheckbox;
+  Label *mpJacobianLabel;
+  QComboBox *mpJacobianComboBox;
+  QGroupBox *mpDasslIdaOptionsGroupBox;
+  QCheckBox *mpRootFindingCheckBox;
+  QCheckBox *mpRestartAfterEventCheckBox;
+  Label *mpInitialStepSizeLabel;
+  QLineEdit *mpInitialStepSizeTextBox;
+  Label *mpMaxStepSizeLabel;
+  QLineEdit *mpMaxStepSizeTextBox;
+  Label *mpMaxIntegrationOrderLabel;
+  QSpinBox *mpMaxIntegrationOrderSpinBox;
   Label *mpCflagsLabel;
   QLineEdit *mpCflagsTextBox;
   Label *mpNumberOfProcessorsLabel;
@@ -117,23 +137,14 @@ private:
   QCheckBox *mpBuildOnlyCheckBox;
   QCheckBox *mpLaunchTransformationalDebuggerCheckBox;
   QCheckBox *mpLaunchAlgorithmicDebuggerCheckBox;
-  // Output Tab
-  QWidget *mpOutputTab;
-  Label *mpNumberofIntervalLabel;
-  QSpinBox *mpNumberofIntervalsSpinBox;
-  Label *mpOutputFormatLabel;
-  QComboBox *mpOutputFormatComboBox;
-  Label *mpFileNameLabel;
-  QLineEdit *mpFileNameTextBox;
-  Label *mpResultFileNameLabel;
-  QLineEdit *mpResultFileNameTextBox;
-  Label *mpResultFileName;
-  Label *mpVariableFilterLabel;
-  QLineEdit *mpVariableFilterTextBox;
-  QCheckBox *mpProtectedVariablesCheckBox;
-  QCheckBox *mpEquidistantTimeGridCheckBox;
-  QCheckBox *mpStoreVariablesAtEventsCheckBox;
-  QCheckBox *mpShowGeneratedFilesCheckBox;
+#if !defined(WITHOUT_OSG)
+  QCheckBox *mpLaunchAnimationCheckBox;
+#endif
+  // Interactive Simulation Tab
+  QWidget *mpInteractiveSimulationTab;
+  // Translation Tab
+  QWidget *mpTranslationTab;
+  TranslationFlagsWidget *mpTranslationFlagsWidget;
   // Simulation Flags Tab
   QWidget *mpSimulationFlagsTab;
   QScrollArea *mpSimulationFlagsTabScrollArea;
@@ -163,62 +174,101 @@ private:
   QComboBox *mpProfilingComboBox;
   QCheckBox *mpCPUTimeCheckBox;
   QCheckBox *mpEnableAllWarningsCheckBox;
+  QGroupBox *mpReconcileGroupBox;
+  Label *mpDataReconciliationInputFileLabel;
+  QLineEdit *mpDataReconciliationInputFileTextBox;
+  QPushButton *mpDataReconciliationInputFileBrowseButton;
+  Label *mpDataReconciliationEpsilonLabel;
+  QLineEdit *mpDataReconciliationEpsilonTextBox;
   QGroupBox *mpLoggingGroupBox;
-  QCheckBox *mpLogDasslSolverCheckBox;
-  QCheckBox *mpLogDebugCheckBox;
-  QCheckBox *mpLogDynamicStateSelectionCheckBox;
-  QCheckBox *mpLogJacobianDynamicStateSelectionCheckBox;
-  QCheckBox *mpLogEventsCheckBox;
-  QCheckBox *mpLogVerboseEventsCheckBox;
-  QCheckBox *mpLogInitializationCheckBox;
-  QCheckBox *mpLogJacobianCheckBox;
-  QCheckBox *mpLogNonLinearSystemsCheckBox;
-  QCheckBox *mpLogVerboseNonLinearSystemsCheckBox;
-  QCheckBox *mpLogJacobianNonLinearSystemsCheckBox;
-  QCheckBox *mpLogResidualsInitializationCheckBox;
-  QCheckBox *mpLogSimulationCheckBox;
-  QCheckBox *mpLogSolverCheckBox;
-  QCheckBox *mpLogFinalSolutionOfInitializationCheckBox;
-  QCheckBox *mpLogStatsCheckBox;
-  QCheckBox *mpLogUtilCheckBox;
-  QCheckBox *mpLogZeroCrossingsCheckBox;
+  QGridLayout *mpLoggingGroupLayout;
   Label *mpAdditionalSimulationFlagsLabel;
   QLineEdit *mpAdditionalSimulationFlagsTextBox;
+  QToolButton *mpSimulationFlagsHelpButton;
+  QGroupBox *mpInteractiveSimulationGroupBox;
+  Label *mpInteractiveSimulationPortLabel;
+  QLineEdit *mpInteractiveSimulationPortNumberTextBox;
+  QCheckBox *mpInteractiveSimulationStepCheckBox;
+  // Output Tab
+  QWidget *mpOutputTab;
+  Label *mpOutputFormatLabel;
+  QComboBox *mpOutputFormatComboBox;
+  QCheckBox *mpSinglePrecisionCheckBox;
+  Label *mpFileNameLabel;
+  QLineEdit *mpFileNameTextBox;
+  Label *mpResultFileNameLabel;
+  QLineEdit *mpResultFileNameTextBox;
+  Label *mpResultFileName;
+  Label *mpVariableFilterLabel;
+  QLineEdit *mpVariableFilterTextBox;
+  QCheckBox *mpProtectedVariablesCheckBox;
+  QCheckBox *mpEquidistantTimeGridCheckBox;
+  QCheckBox *mpStoreVariablesAtEventsCheckBox;
+  QCheckBox *mpShowGeneratedFilesCheckBox;
   // Archived Simulation Flags Tab
   QWidget *mpArchivedSimulationsTab;
   QTreeWidget *mpArchivedSimulationsTreeWidget;
+  // checkboxes
+  QCheckBox *mpSaveExperimentAnnotationCheckBox;
+  QCheckBox *mpSaveSimulationFlagsAnnotationCheckBox;
+  QCheckBox *mpSaveTranslationFlagsAnnotationCheckBox;
+  QCheckBox *mpSimulateCheckBox;
   // buttons
+  QPushButton *mpOkButton;
   QPushButton *mpCancelButton;
-  QPushButton *mpSimulateButton;
   QDialogButtonBox *mpButtonBox;
   QList<SimulationOutputWidget*> mSimulationOutputWidgetsList;
-  LibraryTreeNode *mpLibraryTreeNode;
+  LibraryTreeItem *mpLibraryTreeItem;
   QString mClassName;
   QString mFileName;
   bool mIsReSimulate;
+  // interactive simulation
+  QMap<int, OpcUaClient*> mOpcUaClientsMap;
 
   void setUpForm();
   bool validate();
   void initializeFields(bool isReSimulate, SimulationOptions simulationOptions);
+  void applySimulationOptions(SimulationOptions simulationOptions);
   bool translateModel(QString simulationParameters);
   SimulationOptions createSimulationOptions();
   void createAndShowSimulationOutputWidget(SimulationOptions simulationOptions);
   void showSimulationOutputWidget(SimulationOutputWidget *pSimulationOutputWidget);
-  void saveSimulationOptions();
+  void saveExperimentAnnotation();
+  void saveSimulationFlagsAnnotation();
+  void saveTranslationFlagsAnnotation();
+  void performSimulation();
+  void saveDialogGeometry();
+  void killSimulationProcess(int port);
+  void removeVariablesFromTree(QString className);
+  void terminateSimulationProcess(SimulationOutputWidget *pSimulationOutputWidget);
+  void setInteractiveControls(bool enabled);
 public:
   void reSimulate(SimulationOptions simulationOptions);
   void showAlgorithmicDebugger(SimulationOptions simulationOptions);
   void simulationProcessFinished(SimulationOptions simulationOptions, QDateTime resultFileLastModifiedDateTime);
+  void createOpcUaClient(SimulationOptions simulationOptions);
 public slots:
-  void enableDasslOptions(QString method);
+  void numberOfIntervalsRadioToggled(bool toggle);
+  void intervalRadioToggled(bool toggle);
+  void updateMethodToolTip(int index);
+  void enableDasslIdaOptions(QString method);
   void showIntegrationHelp();
+  void updateJacobianToolTip(int index);
   void buildOnly(bool checked);
+  void interactiveSimulation(bool checked);
   void browseModelSetupFile();
   void browseEquationSystemInitializationFile();
+  void browseDataReconciliationInputFile();
+  void showSimulationFlagsHelp();
   void showArchivedSimulation(QTreeWidgetItem *pTreeWidgetItem);
   void simulate();
+  void reject();
+  void updateInteractiveSimulationCurves();
+  void updateYAxis(double min, double max);
 private slots:
   void resultFileNameChanged(QString text);
+  void simulationStarted();
+  void simulationPaused();
 };
 
 #endif // SIMULATIONDIALOG_H
