@@ -44,7 +44,7 @@
  * - Dr. Henning Kiel - <a href="mailto:henning.kiel@w-hs.de">henning.kiel@w-hs.de</a>
  * - Haris Kapidzic
  * - Abhinn Kothari
- * - Lennart Ochel - <a href="mailto:lennart.ochel@fh-bielefeld.de">lennart.ochel@fh-bielefeld.de</a>
+ * - Lennart Ochel - <a href="mailto:lennart.ochel@liu.se">lennart.ochel@liu.se</a>
  * - Volker Waurich - <a href="mailto:volker.waurich@tu-dresden.de">volker.waurich@tu-dresden.de</a>
  * - Rüdiger Franke
  * - Martin Flehmig
@@ -143,6 +143,14 @@ void printOMEditUsage()
   printf("    files                       List of Modelica files(*.mo) to open.\n");
 }
 
+static int execution_failed()
+{
+  fflush(NULL);
+  fprintf(stderr, "Execution failed!\n");
+  fflush(NULL);
+  return 1;
+}
+
 int main(int argc, char *argv[])
 {
   MMC_INIT();
@@ -173,5 +181,5 @@ int main(int argc, char *argv[])
   OMEditApplication a(argc, argv, threadData);
   return a.exec();
 
-  MMC_CATCH_TOP();
+  MMC_CATCH_TOP(return execution_failed());
 }
