@@ -21,7 +21,7 @@
  * GNU version 3 is obtained from: http://www.gnu.org/copyleft/gpl.html.
  *
  * This program is distributed WITHOUT ANY WARRANTY; without
- * even the implied warranty of  MERCHANTABILITY or FITNESS
+ * even the implied warranty of MERCHANTABILITY or FITNESS
  * FOR A PARTICULAR PURPOSE, EXCEPT AS EXPRESSLY SET FORTH
  * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS OF OSMC-PL.
  *
@@ -30,9 +30,16 @@
  */
 
 #include "ComRef.h"
-#include "Types.h"
-#include "Identifier.h"
 #include <assert.h>
+#include <RegEx.h>
+
+const oms_regex re_ident("^[a-zA-Z][a-zA-Z0-9_]*$");
+
+oms::ComRef::ComRef()
+{
+  cref = new char[1];
+  cref[0] = '\0';
+}
 
 oms::ComRef::ComRef(const std::string& path)
 {
@@ -78,7 +85,7 @@ oms::ComRef oms::ComRef::operator+(const oms::ComRef& rhs) const
 
 bool oms::ComRef::isValidIdent(const std::string& ident)
 {
-  return oms_regex_match(ident, regex_ident);
+  return oms_regex_match(ident, re_ident);
 }
 
 bool oms::ComRef::isValidIdent() const
@@ -91,6 +98,17 @@ bool oms::ComRef::isEmpty() const
   return (cref[0] == '\0');
 }
 
+bool oms::ComRef::isRootOf(ComRef child) const
+{
+  ComRef root(*this);
+  while (!root.isEmpty())
+  {
+    if (child.pop_front() != root.pop_front())
+      return false;
+  }
+  return true;
+}
+
 oms::ComRef oms::ComRef::front() const
 {
   for (int i=0; cref[i]; ++i)
