@@ -21,7 +21,7 @@
  * GNU version 3 is obtained from: http://www.gnu.org/copyleft/gpl.html.
  *
  * This program is distributed WITHOUT ANY WARRANTY; without
- * even the implied warranty of  MERCHANTABILITY or FITNESS
+ * even the implied warranty of MERCHANTABILITY or FITNESS
  * FOR A PARTICULAR PURPOSE, EXCEPT AS EXPRESSLY SET FORTH
  * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS OF OSMC-PL.
  *
@@ -31,6 +31,7 @@
 
 #include "ComponentFMUCS.h"
 
+#include "Flags.h"
 #include "Logging.h"
 #include "Model.h"
 #include "ssd/Tags.h"
@@ -39,7 +40,7 @@
 #include "SystemWC.h"
 #include <fmilib.h>
 #include <JM/jm_portability.h>
-#include <OMSBoost.h>
+#include <OMSFileSystem.h>
 #include <RegEx.h>
 
 oms::ComponentFMUCS::ComponentFMUCS(const ComRef& cref, System* parentSystem, const std::string& fmuPath)
@@ -53,12 +54,6 @@ oms::ComponentFMUCS::~ComponentFMUCS()
   fmi2_import_destroy_dllfmu(fmu);
   fmi2_import_free(fmu);
   fmi_import_free_context(context);
-
-  if (!tempDir.empty() && boost::filesystem::is_directory(tempDir))
-  {
-    boost::filesystem::remove_all(tempDir);
-    logDebug("removed working directory: \"" + tempDir + "\"");
-  }
 }
 
 oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::System* parentSystem, const std::string& fmuPath)
@@ -75,14 +70,12 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::
     return NULL;
   }
 
-  boost::filesystem::path temp_root(parentSystem->getModel()->getTempDirectory());
-  boost::filesystem::path temp_temp = temp_root / "temp";
-  boost::filesystem::path temp_resources = temp_root / "resources";
+  filesystem::path temp_root(parentSystem->getModel()->getTempDirectory());
+  filesystem::path temp_temp = temp_root / "temp";
+  filesystem::path relFMUPath = parentSystem->copyResources() ? (filesystem::path("resources") / (parentSystem->getUniqueID() + "_" + std::string(cref) + ".fmu")) : filesystem::path(fmuPath);
+  filesystem::path absFMUPath = temp_root / relFMUPath;
 
-  boost::filesystem::path relFMUPath = boost::filesystem::path("resources") / (std::string(cref) + ".fmu");
-  boost::filesystem::path absFMUPath = temp_root / relFMUPath;
-
-  ComponentFMUCS* component = new ComponentFMUCS(cref, parentSystem, "resources/" + std::string(cref) + ".fmu");
+  ComponentFMUCS* component = new ComponentFMUCS(cref, parentSystem, relFMUPath.string());
 
   component->callbacks.malloc = malloc;
   component->callbacks.calloc = calloc;
@@ -92,13 +85,15 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::
   component->callbacks.log_level = jm_log_level_all;
   component->callbacks.context = 0;
 
+  // Copy the resource to the temp directory of the model? We don't want have
+  // do copy resources if importing an SSP or snapshot.
   if (parentSystem->copyResources())
-    boost::filesystem::copy_file(boost::filesystem::path(fmuPath), absFMUPath, boost::filesystem::copy_option::overwrite_if_exists);
+    oms_copy_file(filesystem::path(fmuPath), absFMUPath);
 
   // set temp directory
-  boost::filesystem::path tempDir = temp_temp / std::string(cref);
-  component->tempDir = tempDir.string();
-  if (!boost::filesystem::is_directory(tempDir) && !boost::filesystem::create_directory(tempDir))
+  filesystem::path tempDir = temp_temp / relFMUPath.stem();
+  component->setTempDir(tempDir.string());
+  if (!filesystem::is_directory(tempDir) && !filesystem::create_directory(tempDir))
   {
     logError("Creating temp directory for component \"" + std::string(cref) + "\" failed");
     return NULL;
@@ -107,7 +102,7 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::
   component->context = fmi_import_allocate_context(&component->callbacks);
 
   // check version of FMU
-  fmi_version_enu_t version = fmi_import_get_fmi_version(component->context, absFMUPath.string().c_str(), component->tempDir.c_str());
+  fmi_version_enu_t version = fmi_import_get_fmi_version(component->context, absFMUPath.string().c_str(), component->getTempDir().c_str());
   if (fmi_version_2_0_enu != version)
   {
     logError("Unsupported FMI version: " + std::string(fmi_version_to_string(version)));
@@ -116,7 +111,7 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::
   }
 
   // parse modelDescription.xml
-  component->fmu = fmi2_import_parse_xml(component->context, component->tempDir.c_str(), 0);
+  component->fmu = fmi2_import_parse_xml(component->context, component->getTempDir().c_str(), 0);
   if (!component->fmu)
   {
     logError("Error parsing modelDescription.xml");
@@ -475,6 +470,8 @@ oms_status_enu_t oms::ComponentFMUCS::initialize()
 
 oms_status_enu_t oms::ComponentFMUCS::terminate()
 {
+  freeState();
+
   fmi2_status_t fmistatus = fmi2_import_terminate(fmu);
   if (fmi2_status_ok != fmistatus)
     return logError_Termination(getCref());
@@ -572,7 +569,7 @@ oms_status_enu_t oms::ComponentFMUCS::getBoolean(const ComRef& cref, bool& value
   }
 
   if (!fmu || j < 0)
-    return oms_status_error;
+    return logError_UnknownSignal(getFullCref() + cref);
 
   fmi2_value_reference_t vr = allVariables[j].getValueReference();
   return getBoolean(vr, value);
@@ -602,12 +599,23 @@ oms_status_enu_t oms::ComponentFMUCS::getInteger(const ComRef& cref, int& value)
   }
 
   if (!fmu || j < 0)
-    return oms_status_error;
+    return logError_UnknownSignal(getFullCref() + cref);
 
   fmi2_value_reference_t vr = allVariables[j].getValueReference();
   return getInteger(vr, value);
 }
 
+oms::Variable* oms::ComponentFMUCS::getVariable(const ComRef& cref)
+{
+  CallClock callClock(clock);
+  for (size_t i=0; i < allVariables.size(); i++)
+    if (allVariables[i].getCref() == cref)
+      return &allVariables[i];
+
+  logError_UnknownSignal(getFullCref() + cref);
+  return NULL;
+}
+
 oms_status_enu_t oms::ComponentFMUCS::getReal(const fmi2_value_reference_t& vr, double& value)
 {
   CallClock callClock(clock);
@@ -620,6 +628,29 @@ oms_status_enu_t oms::ComponentFMUCS::getReal(const fmi2_value_reference_t& vr,
   if (std::isinf(value))
     return logError("getReal returned +/-inf");
 
+  // Check for FIB
+  auto block = fib.find(vr);
+  if (block != fib.end())
+  {
+    switch(block->second.faultType)
+    {
+      case oms_fault_type_bias:   // y = y.$original + faultValue
+        value += block->second.faultValue;
+        break;
+
+      case oms_fault_type_gain:   // y = y.$original * faultValue
+        value *= block->second.faultValue;
+        break;
+
+      case oms_fault_type_const:  // y = faultValue
+        value = block->second.faultValue;
+        break;
+
+      default:
+        return logError("Unhandled fault injection block");
+    }
+  }
+
   return oms_status_ok;
 }
 
@@ -637,15 +668,16 @@ oms_status_enu_t oms::ComponentFMUCS::getReal(const ComRef& cref, double& value)
   }
 
   if (!fmu || j < 0)
-    return oms_status_error;
+    return logError_UnknownSignal(getFullCref() + cref);
 
   fmi2_value_reference_t vr = allVariables[j].getValueReference();
   return getReal(vr, value);
 }
 
-oms_status_enu_t oms::ComponentFMUCS::getRealOutputDerivative(const ComRef& cref, double*& value)
+oms_status_enu_t oms::ComponentFMUCS::getRealOutputDerivative(const ComRef& cref, SignalDerivative& der)
 {
   CallClock callClock(clock);
+
   int j=-1;
   for (size_t i = 0; i < allVariables.size(); i++)
   {
@@ -657,48 +689,39 @@ oms_status_enu_t oms::ComponentFMUCS::getRealOutputDerivative(const ComRef& cref
   }
 
   if (!fmu || j < 0)
-    return oms_status_error;
-
-  fmi2_value_reference_t vr = allVariables[j].getValueReference();
-  fmi2_integer_t order = getFMUInfo()->getMaxOutputDerivativeOrder();
-  if (order > 0)
-  {
-    if (fmi2_status_ok != fmi2_import_get_real_output_derivatives(fmu, &vr, 1, &order, value))
-      return oms_status_error;
-
-    if (std::isnan(value[0]))
-      return logError("getRealOutputDerivative returned NAN");
-    if (std::isinf(value[0]))
-      return logError("getRealOutputDerivative returned +/-inf");
-  }
+    return logError_UnknownSignal(getFullCref() + cref);
 
+  der = SignalDerivative(getFMUInfo()->getMaxOutputDerivativeOrder(), fmu, allVariables[j].getValueReference());
   return oms_status_ok;
 }
 
-oms_status_enu_t oms::ComponentFMUCS::setRealInputDerivative(const ComRef& cref, double* value, unsigned int order)
+oms_status_enu_t oms::ComponentFMUCS::setRealInputDerivative(const ComRef& cref, const SignalDerivative& der)
 {
   CallClock callClock(clock);
+
+  if (!getFMUInfo()->getCanInterpolateInputs())
+    return oms_status_ok;
+
   int j=-1;
   for (size_t i = 0; i < allVariables.size(); i++)
   {
-    if (allVariables[i].getCref() == cref && allVariables[i].isTypeReal() && allVariables[i].isInput())
+    if (allVariables[i].getCref() == cref)
     {
-      j = i;
-      break;
+      if (allVariables[i].isTypeReal() && allVariables[i].isInput())
+      {
+        j = i;
+        break;
+      }
+      else
+        return logError_OnlyForRealInputs(getFullCref() + cref);
     }
   }
 
   if (!fmu || j < 0)
-    return oms_status_error;
+    return logError_UnknownSignal(getFullCref() + cref);
 
   fmi2_value_reference_t vr = allVariables[j].getValueReference();
-  if (order > 0)
-  {
-    if (fmi2_status_ok != fmi2_import_set_real_input_derivatives(fmu, &vr, 1, (fmi2_integer_t*)&order, value))
-      return oms_status_error;
-  }
-
-  return oms_status_ok;
+  return der.setRealInputDerivatives(fmu, vr);
 }
 
 oms_status_enu_t oms::ComponentFMUCS::setBoolean(const ComRef& cref, bool value)
@@ -715,7 +738,7 @@ oms_status_enu_t oms::ComponentFMUCS::setBoolean(const ComRef& cref, bool value)
   }
 
   if (!fmu || j < 0)
-    return oms_status_error;
+    return logError_UnknownSignal(getFullCref() + cref);
 
   if (oms_modelState_virgin == getModel()->getModelState())
     booleanStartValues[allVariables[j].getCref()] = value;
@@ -744,7 +767,7 @@ oms_status_enu_t oms::ComponentFMUCS::setInteger(const ComRef& cref, int value)
   }
 
   if (!fmu || j < 0)
-    return oms_status_error;
+    return logError_UnknownSignal(getFullCref() + cref);
 
   if (oms_modelState_virgin == getModel()->getModelState())
     integerStartValues[allVariables[j].getCref()] = value;
@@ -772,7 +795,7 @@ oms_status_enu_t oms::ComponentFMUCS::setReal(const ComRef& cref, double value)
   }
 
   if (!fmu || j < 0)
-    return oms_status_error;
+    return logError_UnknownSignal(getFullCref() + cref);
 
   if (getModel()->validState(oms_modelState_virgin|oms_modelState_enterInstantiation|oms_modelState_instantiated))
     if (allVariables[j].isCalculated() || allVariables[j].isIndependent())
@@ -794,7 +817,11 @@ oms_status_enu_t oms::ComponentFMUCS::registerSignalsForResultFile(ResultWriter&
 {
   resultFileMapping.clear();
 
-  clock_id = resultFile.addSignal(std::string(getFullCref() + ComRef("$wallTime")), "wall-clock time [s]", SignalType_REAL);
+  if (Flags::WallTime())
+    clock_id = resultFile.addSignal(std::string(getFullCref() + ComRef("$wallTime")), "wall-clock time [s]", SignalType_REAL);
+  else
+    clock_id = 0;
+
   for (unsigned int i=0; i<allVariables.size(); ++i)
   {
     if (!exportVariables[i])
@@ -852,9 +879,13 @@ oms_status_enu_t oms::ComponentFMUCS::registerSignalsForResultFile(ResultWriter&
 oms_status_enu_t oms::ComponentFMUCS::updateSignals(ResultWriter& resultWriter)
 {
   CallClock callClock(clock);
-  SignalValue_t wallTime;
-  wallTime.realValue = clock.getElapsedWallTime();
-  resultWriter.updateSignal(clock_id, wallTime);
+
+  if (clock_id)
+  {
+    SignalValue_t wallTime;
+    wallTime.realValue = clock.getElapsedWallTime();
+    resultWriter.updateSignal(clock_id, wallTime);
+  }
 
   for (auto const &it : resultFileMapping)
   {
@@ -922,3 +953,56 @@ oms_status_enu_t oms::ComponentFMUCS::removeSignalsFromResults(const char* regex
 
   return oms_status_ok;
 }
+
+oms_status_enu_t oms::ComponentFMUCS::saveState()
+{
+  fmi2_status_t fmistatus = fmi2_import_get_fmu_state(fmu, &fmuState);
+  if (fmi2_status_ok != fmistatus) return logError_FMUCall("fmi2_import_get_fmu_state", this);
+  fmuStateTime = time;
+
+  return oms_status_ok;
+}
+
+oms_status_enu_t oms::ComponentFMUCS::freeState()
+{
+  if (!fmuState)
+    return oms_status_warning;
+
+  fmi2_status_t fmistatus = fmi2_import_free_fmu_state(fmu, &fmuState);
+  fmuState = NULL;
+  if (fmi2_status_ok != fmistatus) return logError_FMUCall("fmi2_import_free_fmu_state", this);
+
+  return oms_status_ok;
+}
+
+oms_status_enu_t oms::ComponentFMUCS::restoreState()
+{
+  fmi2_status_t fmistatus = fmi2_import_set_fmu_state(fmu, fmuState);
+  if (fmi2_status_ok != fmistatus) return logError_FMUCall("fmi2_import_set_fmu_state", this);
+  time = fmuStateTime;
+
+  return oms_status_ok;
+}
+
+oms_status_enu_t oms::ComponentFMUCS::setFaultInjection(const oms::ComRef& signal, oms_fault_type_enu_t faultType, double faultValue)
+{
+  Variable* var = getVariable(signal);
+
+  if (!var || !var->isOutput())
+    return oms_status_error;
+
+  // remove fib?
+  if ((faultType == oms_fault_type_bias && faultValue == 0.0) ||
+      (faultType == oms_fault_type_gain && faultValue == 1.0))
+  {
+    fib.erase(var->getValueReference());
+  }
+  else
+  {
+    oms_fault_type_t ft;
+    ft.faultType = faultType;
+    ft.faultValue = faultValue;
+    fib[var->getValueReference()] = ft;
+  }
+  return oms_status_ok;
+}
