@@ -21,7 +21,7 @@
  * GNU version 3 is obtained from: http://www.gnu.org/copyleft/gpl.html.
  *
  * This program is distributed WITHOUT ANY WARRANTY; without
- * even the implied warranty of  MERCHANTABILITY or FITNESS
+ * even the implied warranty of MERCHANTABILITY or FITNESS
  * FOR A PARTICULAR PURPOSE, EXCEPT AS EXPRESSLY SET FORTH
  * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS OF OSMC-PL.
  *
@@ -34,7 +34,6 @@
 
 #include "Component.h"
 #include "ComRef.h"
-#include "Option.h"
 #include "ResultWriter.h"
 #include "Variable.h"
 #include <fmilib.h>
@@ -68,6 +67,8 @@ namespace oms
     oms_status_enu_t initializeDependencyGraph_initialUnknowns();
     oms_status_enu_t initializeDependencyGraph_outputs();
 
+    Variable* getVariable(const ComRef& cref);
+
     oms_status_enu_t getBoolean(const ComRef& cref, bool& value);
     oms_status_enu_t getBoolean(const fmi2_value_reference_t& vr, bool& value);
     oms_status_enu_t getInteger(const ComRef& cref, int& value);
@@ -78,14 +79,27 @@ namespace oms
     oms_status_enu_t setInteger(const ComRef& cref, int value);
     oms_status_enu_t setReal(const ComRef& cref, double value);
 
-    oms_status_enu_t getRealOutputDerivative(const ComRef& cref, double*& value);
-    oms_status_enu_t setRealInputDerivative(const ComRef& cref, double* value, unsigned int order);
+    oms_status_enu_t setFmuTime(double time) {this->time = time; return oms_status_ok;}
+    fmi2_import_t* getFMU() {return fmu;}
+    std::vector<Variable> getAllVariables() {return allVariables;}
+    std::vector<Variable> getInputs() {return inputs;}
+    std::vector<Variable> getOutputs() {return outputs;}
+
+    oms_status_enu_t getRealOutputDerivative(const ComRef& cref, SignalDerivative& der);
+    oms_status_enu_t setRealInputDerivative(const ComRef& cref, const SignalDerivative& der);
 
     oms_status_enu_t registerSignalsForResultFile(ResultWriter& resultFile);
     oms_status_enu_t updateSignals(ResultWriter& resultWriter);
     oms_status_enu_t addSignalsToResults(const char* regex);
     oms_status_enu_t removeSignalsFromResults(const char* regex);
 
+    bool getCanGetAndSetState() {return getFMUInfo()->getCanGetAndSetFMUstate();}
+    oms_status_enu_t saveState();
+    oms_status_enu_t freeState();
+    oms_status_enu_t restoreState();
+
+    oms_status_enu_t setFaultInjection(const ComRef& signal, oms_fault_type_enu_t faultType, double faultValue);
+
   protected:
     ComponentFMUCS(const ComRef& cref, System* parentSystem, const std::string& fmuPath);
 
@@ -98,7 +112,6 @@ namespace oms
     fmi2_callback_functions_t callbackFunctions;
     fmi_import_context_t* context = NULL;
     fmi2_import_t* fmu = NULL;
-    std::string tempDir;
 
     FMUInfo fmuInfo;
 
@@ -115,6 +128,11 @@ namespace oms
     std::unordered_map<unsigned int /*result file var ID*/, unsigned int /*allVariables ID*/> resultFileMapping;
 
     double time;
+
+    fmi2_FMU_state_t fmuState = NULL;
+    double fmuStateTime;
+
+    std::map<fmi2_value_reference_t, oms_fault_type_t> fib;  ///< fault injection blocks
   };
 }
 
