@@ -21,7 +21,7 @@
  * GNU version 3 is obtained from: http://www.gnu.org/copyleft/gpl.html.
  *
  * This program is distributed WITHOUT ANY WARRANTY; without
- * even the implied warranty of  MERCHANTABILITY or FITNESS
+ * even the implied warranty of MERCHANTABILITY or FITNESS
  * FOR A PARTICULAR PURPOSE, EXCEPT AS EXPRESSLY SET FORTH
  * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS OF OSMC-PL.
  *
@@ -39,8 +39,9 @@
 #include "System.h"
 #include "Component.h"
 
-#include <OMSBoost.h>
+#include <OMSFileSystem.h>
 #include <minizip.h>
+#include <thread>
 
 oms::Model::Model(const oms::ComRef& cref, const std::string& tempDir)
   : cref(cref), tempDir(tempDir), resultFilename(std::string(cref) + "_res.mat")
@@ -58,6 +59,23 @@ oms::Model::~Model()
 
   if (system)
     delete system;
+
+  // delete temp directory
+  if (Flags::DeleteTempFiles())
+  {
+    if (!tempDir.empty() && filesystem::is_directory(tempDir))
+    {
+      try
+      {
+        filesystem::remove_all(tempDir);
+        logDebug("removed temp directory: \"" + tempDir + "\"");
+      }
+      catch (const std::exception& e)
+      {
+        logWarning("temp directory \"" + tempDir + "\" couldn't be removed\n" + e.what());
+      }
+    }
+  }
 }
 
 oms::Model* oms::Model::NewModel(const oms::ComRef& cref)
@@ -68,23 +86,23 @@ oms::Model* oms::Model::NewModel(const oms::ComRef& cref)
     return NULL;
   }
 
-  boost::filesystem::path tempDir = (boost::filesystem::path(Scope::GetInstance().getTempDirectory().c_str()) / oms_unique_path(std::string(cref))).string();
-  if (boost::filesystem::is_directory(tempDir))
+  filesystem::path tempDir = (filesystem::path(Scope::GetInstance().getTempDirectory().c_str()) / oms_unique_path(std::string(cref))).string();
+  if (filesystem::is_directory(tempDir))
   {
     logError("Unique temp directory does already exist. Clean up the temp directory \"" + Scope::GetInstance().getTempDirectory() + "\" and try again.");
     return NULL;
   }
-  if (!boost::filesystem::create_directory(tempDir))
+  if (!filesystem::create_directory(tempDir))
   {
     logError("Failed to create unique temp directory for the model \"" + std::string(cref) + "\"");
     return NULL;
   }
-  if (!boost::filesystem::create_directory(tempDir / "temp"))
+  if (!filesystem::create_directory(tempDir / "temp"))
   {
     logError("Failed to create temp directory for the model \"" + std::string(cref) + "\"");
     return NULL;
   }
-  if (!boost::filesystem::create_directory(tempDir / "resources"))
+  if (!filesystem::create_directory(tempDir / "resources"))
   {
     logError("Failed to create resources directory for the model \"" + std::string(cref) + "\"");
     return NULL;
@@ -103,6 +121,47 @@ oms_status_enu_t oms::Model::rename(const oms::ComRef& cref)
   return oms_status_ok;
 }
 
+oms_status_enu_t oms::Model::loadSnapshot(const char* snapshot)
+{
+  pugi::xml_document doc;
+  pugi::xml_parse_result result = doc.load(snapshot);
+  if (!result)
+    return logError("loading snapshot failed (" + std::string(result.description()) + ")");
+
+  const pugi::xml_node node = doc.document_element(); // ssd:SystemStructureDescription
+
+  ComRef new_cref = ComRef(node.attribute("name").as_string());
+  std::string ssdVersion = node.attribute("version").as_string();
+
+  if (new_cref != getCref())
+    return logError("this API cannot be used to rename a model");
+
+  if (ssdVersion != "Draft20180219")
+    logWarning("Unknown SSD version: " + ssdVersion);
+
+  System* old_root_system = system;
+  system = NULL;
+
+  bool old_copyResources = copyResources();
+  copyResources(false);
+  oms_status_enu_t status = importFromSSD(node);
+  copyResources(old_copyResources);
+
+  if (oms_status_ok != status)
+  {
+    system = old_root_system;
+    return logError("loading snapshot failed");
+  }
+
+  if (old_root_system)
+  {
+    delete old_root_system;
+    old_root_system = NULL;
+  }
+
+  return oms_status_ok;
+}
+
 oms::System* oms::Model::getSystem(const oms::ComRef& cref)
 {
   if (!system)
@@ -307,7 +366,7 @@ oms_status_enu_t oms::Model::exportToFile(const std::string& filename) const
   pugi::xml_node node = doc.append_child(oms::ssd::ssd_system_structure_description);
   exportToSSD(node);
 
-  boost::filesystem::path ssdPath = boost::filesystem::path(tempDir) / "SystemStructure.ssd";
+  filesystem::path ssdPath = filesystem::path(tempDir) / "SystemStructure.ssd";
   if (!doc.save_file(ssdPath.string().c_str()))
     return logError("failed to export \"" + ssdPath.string() + "\" (for model \"" + std::string(this->getCref()) + "\")");
 
@@ -337,8 +396,8 @@ oms_status_enu_t oms::Model::exportToFile(const std::string& filename) const
   delete[] argv;
   Scope::GetInstance().setWorkingDirectory(cd);
 
-  boost::filesystem::path full_path = boost::filesystem::path(tempDir) / "temp/model.ssp";
-  boost::filesystem::copy_file(full_path, boost::filesystem::path(filename), boost::filesystem::copy_option::overwrite_if_exists);
+  filesystem::path full_path = filesystem::path(tempDir) / "temp/model.ssp";
+  oms_copy_file(full_path, filesystem::path(filename));
 
   return oms_status_ok;
 }
@@ -377,6 +436,16 @@ oms_status_enu_t oms::Model::instantiate()
   if (!system)
     return logError("Model doesn't contain a system");
 
+  // initialize thread pool
+  int numThreads = Flags::NumProcs();
+  if (numThreads != 1)
+  {
+    if (numThreads > std::thread::hardware_concurrency() || 0 == numThreads)
+      numThreads = std::thread::hardware_concurrency();
+    pool = new ctpl::thread_pool(numThreads);
+    logInfo("Initialized thread pool for model " + std::string(getCref()) + " with " + std::to_string(numThreads) + " threads");
+  }
+
   modelState = oms_modelState_enterInstantiation;
   if (oms_status_ok != system->instantiate())
   {
@@ -530,6 +599,12 @@ oms_status_enu_t oms::Model::terminate()
   if (oms_status_ok != system->terminate())
     return logError_Termination(system->getFullCref());
 
+  if (useThreadPool())
+  {
+    delete pool;
+    pool = nullptr;
+  }
+
   if (resultFile)
   {
     delete resultFile;
@@ -580,12 +655,17 @@ oms_status_enu_t oms::Model::registerSignalsForResultFile()
   return oms_status_ok;
 }
 
-oms_status_enu_t oms::Model::emit(double time, bool force)
+oms_status_enu_t oms::Model::emit(double time, bool force, bool* emitted)
 {
+  if (emitted)
+    *emitted = false;
+
   if (!resultFile)
     return oms_status_ok;
   if (!force && time < lastEmit + loggingInterval)
     return oms_status_ok;
+  if (!force && time <= lastEmit)
+    return oms_status_ok;
 
   if (system)
     if (oms_status_ok != system->updateSignals(*resultFile))
@@ -593,6 +673,8 @@ oms_status_enu_t oms::Model::emit(double time, bool force)
 
   resultFile->emit(time);
   lastEmit = time;
+  if (emitted)
+    *emitted = true;
   return oms_status_ok;
 }
 
