@@ -21,7 +21,7 @@
  * GNU version 3 is obtained from: http://www.gnu.org/copyleft/gpl.html.
  *
  * This program is distributed WITHOUT ANY WARRANTY; without
- * even the implied warranty of  MERCHANTABILITY or FITNESS
+ * even the implied warranty of MERCHANTABILITY or FITNESS
  * FOR A PARTICULAR PURPOSE, EXCEPT AS EXPRESSLY SET FORTH
  * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS OF OSMC-PL.
  *
@@ -34,6 +34,7 @@
 #include "Component.h"
 #include "ComRef.h"
 #include "Element.h"
+#include "ExternalModelInfo.h"
 #include "Flags.h"
 #include "FMUInfo.h"
 #include "Logging.h"
@@ -41,14 +42,16 @@
 #include "Model.h"
 #include "ResultReader.h"
 #include "Scope.h"
+#include "SignalDerivative.h"
 #include "System.h"
+#include "SystemWC.h"
 #if !defined(NO_TLM)
   #include "SystemTLM.h"
   #include "TLMBusConnector.h"
+  #include "ExternalModel.h"
 #endif
 #include "Types.h"
 #include "Version.h"
-#include <boost/filesystem.hpp>
 #include <miniunz.h>
 #include <pugixml.hpp>
 #include <string>
@@ -59,6 +62,13 @@
 #include <mutex>
 #include <condition_variable>
 
+#if defined(OMS_STATIC)
+extern "C"
+{
+  #include <OMSimulatorLua.c>
+}
+#endif
+
 const char* oms_getVersion()
 {
   return oms_git_version;
@@ -112,7 +122,9 @@ oms_status_enu_t oms_setWorkingDirectory(const char* newWorkingDir)
 
 oms_status_enu_t oms_newModel(const char* cref)
 {
-  return oms::Scope::GetInstance().newModel(oms::ComRef(cref));
+  if (!oms::Scope::GetInstance().newModel(oms::ComRef(cref)))
+    return oms_status_error;
+  return oms_status_ok;
 }
 
 oms_status_enu_t oms_rename(const char* cref_, const char* newCref_)
@@ -155,19 +167,35 @@ oms_status_enu_t oms_delete(const char* cref)
   oms::ComRef tail(cref);
   oms::ComRef front = tail.pop_front();
 
+  if (tail.isEmpty())
+    return oms::Scope::GetInstance().deleteModel(front);
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+  return model->delete_(tail);
+}
+
+oms_status_enu_t oms_export(const char* cref, const char* filename)
+{
+  return oms::Scope::GetInstance().exportModel(oms::ComRef(cref), std::string(filename));
+}
+
+oms_status_enu_t oms_faultInjection(const char* signal, oms_fault_type_enu_t faultType, double faultValue)
+{
+  oms::ComRef tail(signal);
+  oms::ComRef front = tail.pop_front();
+
   oms::Model* model = oms::Scope::GetInstance().getModel(front);
   if (!model)
     return logError_ModelNotInScope(front);
 
-  if (tail.isEmpty())
-    return oms::Scope::GetInstance().deleteModel(front);
-  else
-    return model->delete_(tail);
-}
+  front = tail.pop_front();
+  oms::System* system = model->getSystem(front);
+  if (!system)
+    return logError_SystemNotInModel(model->getCref(), front);
 
-oms_status_enu_t oms_export(const char* cref_, const char* filename)
-{
-  return oms::Scope::GetInstance().exportModel(oms::ComRef(cref_), std::string(filename));
+  return system->setFaultInjection(tail, faultType, faultValue);
 }
 
 oms_status_enu_t oms_importFile(const char* filename, char** cref)
@@ -201,12 +229,18 @@ oms_status_enu_t oms_listUnconnectedConnectors(const char* cref_, char** content
   return system->listUnconnectedConnectors(contents);
 }
 
-oms_status_enu_t oms_parseModelName(const char* contents, char** cref)
+oms_status_enu_t oms_loadSnapshot(const char* cref_, const char* snapshot)
 {
-  return logError_NotImplemented;
+  oms::ComRef cref(cref_);
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(cref);
+  if (!model)
+    return logError_ModelNotInScope(cref);
+
+  return model->loadSnapshot(snapshot);
 }
 
-oms_status_enu_t oms_importString(const char* contents, char** cref)
+oms_status_enu_t oms_parseModelName(const char* contents, char** cref)
 {
   return logError_NotImplemented;
 }
@@ -234,9 +268,9 @@ void oms_freeMemory(void* obj)
     free(obj);
 }
 
-oms_status_enu_t oms_getElement(const char* cref_, oms_element_t** element)
+oms_status_enu_t oms_getElement(const char* cref, oms_element_t** element)
 {
-  return oms::Scope::GetInstance().getElement(oms::ComRef(cref_), reinterpret_cast<oms::Element**>(element));
+  return oms::Scope::GetInstance().getElement(oms::ComRef(cref), reinterpret_cast<oms::Element**>(element));
 }
 
 oms_status_enu_t oms_getElements(const char* cref, oms_element_t*** elements)
@@ -244,6 +278,24 @@ oms_status_enu_t oms_getElements(const char* cref, oms_element_t*** elements)
   return oms::Scope::GetInstance().getElements(oms::ComRef(cref), reinterpret_cast<oms::Element***>(elements));
 }
 
+oms_status_enu_t oms_getFixedStepSize(const char* cref, double* stepSize)
+{
+  oms::ComRef tail(cref);
+  oms::ComRef front = tail.pop_front();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  front = tail.pop_front();
+  oms::System* system = model->getSystem(front);
+  if (!system)
+    return logError_SystemNotInModel(model->getCref(), front);
+
+  system->getStepSize(NULL, NULL, stepSize);
+  return oms_status_ok;
+}
+
 oms_status_enu_t oms_setElementGeometry(const char* cref, const ssd_element_geometry_t* geometry)
 {
   if (!geometry)
@@ -799,6 +851,37 @@ oms_status_enu_t oms_getSubModelPath(const char* cref, char** path)
   return oms_status_ok;
 }
 
+oms_status_enu_t oms_getExternalModelInfo(const char* cref, const oms_external_tlm_model_info_t** externalModelInfo)
+{
+#if !defined(NO_TLM)
+  oms::ComRef tail(cref);
+  oms::ComRef front = tail.pop_front();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  front = tail.pop_front();
+  oms::System* system = model->getSystem(front);
+  if (!system)
+    return logError_SystemNotInModel(model->getCref(), front);
+
+  oms::Component* component = system->getComponent(tail);
+  if (!component)
+    return logError_ComponentNotInSystem(system, tail);
+
+  if (component->getType() == oms_component_external)
+  {
+    *reinterpret_cast<const oms::ExternalModelInfo**>(externalModelInfo) = reinterpret_cast<oms::ExternalModel*>(component)->getExternalModelInfo();
+    return oms_status_ok;
+  }
+
+  return oms_status_error;
+#else
+  return LOG_NO_TLM();
+#endif
+}
+
 oms_status_enu_t oms_getFMUInfo(const char* cref, const oms_fmu_info_t** fmuInfo)
 {
   oms::ComRef tail(cref);
@@ -960,6 +1043,11 @@ oms_status_enu_t oms_RunFile(const char* filename_)
     status = oms_newModel(modelName.c_str());
     if (oms_status_ok != status) return logError("oms_newModel failed");
 
+    oms::Model* model = oms::Scope::GetInstance().getModel(oms::ComRef(modelName));
+    if (!model)
+      return logError_ModelNotInScope(oms::ComRef(modelName));
+    model->setIsolatedFMUModel();
+
     if (kind == oms_fmi_kind_me_and_cs)
       status = oms_addSystem(systemName.c_str(), oms::Flags::DefaultModeIsCS() ? oms_system_wc : oms_system_sc);
     else
@@ -1001,6 +1089,23 @@ oms_status_enu_t oms_RunFile(const char* filename_)
     oms_delete(cref);
     return status;
   }
+  else if (type == ".lua")
+  {
+#if defined(OMS_STATIC)
+    lua_State *L = luaL_newstate();
+    luaL_openlibs(L);
+    luaopen_OMSimulatorLua(L);
+    if (luaL_loadfile(L, filename.c_str()))
+      return logError(lua_tostring(L, -1));
+
+    if (lua_pcall(L, 0, 0, 0))
+      return logError(lua_tostring(L, -1));
+
+    lua_close(L);
+#else
+    return logError("Lua is only supported in the static version of OMSimulatorLib");
+#endif
+  }
   else
     return logError("Not able to process file '" + filename + "'\nUse OMSimulator --help for more information.");
 
@@ -1034,7 +1139,7 @@ oms_status_enu_t oms_setTLMSocketData(const char *cref, const char *address, int
     return logError_SystemNotInModel(model->getCref(), front);
 
   if (system->getType() != oms_system_tlm)
-    return logError_OnlyForTlmSystem;
+    return logError_OnlyForSystemTLM;
 
   oms::SystemTLM* tlmsystem = reinterpret_cast<oms::SystemTLM*>(system);
   return tlmsystem->setSocketData(address, managerPort, monitorPort);
@@ -1059,7 +1164,7 @@ oms_status_enu_t oms_setTLMPositionAndOrientation(const char *cref, double x1, d
     return logError_SystemNotInModel(model->getCref(), front);
 
   if (system->getType() != oms_system_tlm)
-    return logError_OnlyForTlmSystem;
+    return logError_OnlyForSystemTLM;
 
   oms::SystemTLM* tlmsystem = reinterpret_cast<oms::SystemTLM*>(system);
   std::vector<double> x = {x1,x2,x3};
@@ -1173,6 +1278,27 @@ oms_status_enu_t oms_setReal(const char* cref, double value)
   return system->setReal(tail, value);
 }
 
+oms_status_enu_t oms_setRealInputDerivative(const char* cref, double value)
+{
+  oms::ComRef tail(cref);
+  oms::ComRef front = tail.pop_front();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  front = tail.pop_front();
+  oms::System* system = model->getSystem(front);
+  if (!system)
+    return logError_SystemNotInModel(model->getCref(), front);
+
+  if (system->getType() != oms_system_wc)
+    return logError_OnlyForSystemWC;
+
+  oms::SystemWC* systemWC = reinterpret_cast<oms::SystemWC*>(system);
+  return systemWC->setRealInputDerivative(tail, oms::SignalDerivative(value));
+}
+
 oms_status_enu_t oms_setInteger(const char* cref, int value)
 {
   oms::ComRef tail(cref);
@@ -1412,7 +1538,7 @@ oms_status_enu_t oms_getVariableStepSize(const char* cref, double* initialStepSi
   if (!system)
     return logError_SystemNotInModel(model->getCref(), front);
 
-  system->getVariableStepSize(initialStepSize, minimumStepSize, maximumStepSize);
+  system->getStepSize(initialStepSize, minimumStepSize, maximumStepSize);
   return oms_status_ok;
 }
 
@@ -1503,3 +1629,87 @@ oms_status_enu_t oms_setTolerance(const char* cref, double absoluteTolerance, do
 
   return logError_SystemNotInModel(model->getCref(), front);
 }
+
+oms_status_enu_t oms_addEventIndicator(const char* cref)
+{
+  oms::ComRef tail(cref);
+  oms::ComRef front = tail.pop_front();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  front = tail.pop_front();
+  oms::System* system = model->getSystem(front);
+  if (!system)
+    return logError_SystemNotInModel(model->getCref(), front);
+
+  if (system->getType() != oms_system_wc)
+    return logError_OnlyForSystemWC;
+
+  oms::SystemWC* systemWC = reinterpret_cast<oms::SystemWC*>(system);
+  return systemWC->addEventIndicator(tail);
+}
+
+oms_status_enu_t oms_addTimeIndicator(const char* cref)
+{
+  oms::ComRef tail(cref);
+  oms::ComRef front = tail.pop_front();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  front = tail.pop_front();
+  oms::System* system = model->getSystem(front);
+  if (!system)
+    return logError_SystemNotInModel(model->getCref(), front);
+
+  if (system->getType() != oms_system_wc)
+    return logError_OnlyForSystemWC;
+
+  oms::SystemWC* systemWC = reinterpret_cast<oms::SystemWC*>(system);
+  return systemWC->addTimeIndicator(tail);
+}
+
+oms_status_enu_t oms_addStaticValueIndicator(const char* cref, double lower, double upper, double stepSize)
+{
+  oms::ComRef tail(cref);
+  oms::ComRef front = tail.pop_front();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  front = tail.pop_front();
+  oms::System* system = model->getSystem(front);
+  if (!system)
+    return logError_SystemNotInModel(model->getCref(), front);
+
+  if (system->getType() != oms_system_wc)
+    return logError_OnlyForSystemWC;
+
+  oms::SystemWC* systemWC = reinterpret_cast<oms::SystemWC*>(system);
+  return systemWC->addStaticValueIndicator(tail, lower, upper, stepSize);
+}
+
+oms_status_enu_t oms_addDynamicValueIndicator(const char* cref, const char* lower, const char* upper, double stepSize)
+{
+  oms::ComRef tail(cref);
+  oms::ComRef front = tail.pop_front();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  front = tail.pop_front();
+  oms::System* system = model->getSystem(front);
+  if (!system)
+    return logError_SystemNotInModel(model->getCref(), front);
+
+  if (system->getType() != oms_system_wc)
+    return logError_OnlyForSystemWC;
+
+  oms::SystemWC* systemWC = reinterpret_cast<oms::SystemWC*>(system);
+  return systemWC->addDynamicValueIndicator(tail, oms::ComRef(lower), oms::ComRef(upper), stepSize);
+}
